"""Window for selecting the backup to restore from."""
from datetime import datetime
import wx
from kvalikirstu2 import argument_parser
from kvalikirstu2 import gui_utils
from kvalikirstu2 import progress_dialog
from kvalikirstu2.localization import _


class BackupSelectWindow(wx.Dialog):
    """The backup selecting window."""
    def __init__(self, model):
        """Initializes the window.

        :param model: The GUIModel instance.

        """
        super().__init__(parent=None, style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER)
        self.model = model
        self.timestamp_map = {}
        self.args = argument_parser.get_args()
        self.timestamps = model.get_backups()
        self.panel = wx.ScrolledWindow(self)
        self.sizer = wx.GridBagSizer(hgap=5, vgap=5)

        self.list_ctrl = wx.ListCtrl(self.panel, style=wx.LC_REPORT)
        self.list_ctrl.InsertColumn(0, _("Timestamp"), width=300)
        self.sizer.Add(self.list_ctrl, pos=(0, 0), span=(3, 4), flag=wx.EXPAND)

        self.init_window()

    def init_window(self):
        """Initializes the window."""
        self.panel.SetScrollbars(1, 1, 1, 1)
        self._add_timestamps()
        self._add_buttons()

        self.sizer.AddGrowableCol(0)
        self.sizer.AddGrowableRow(1)

        gui_utils.add_border_and_fit(self.panel, self.sizer, self)
        self.SetSize(400, 300)
        self.SetTitle(_("Select a backup to restore to"))

    def add_timestamp(self, timestamp, index):
        """Add timestamp to the frame.

        :param str timestamp: The timestamp.
        :para int index: Index of the timestamp.

        """
        self.list_ctrl.InsertItem(index, str(datetime.fromtimestamp(timestamp)))

    def _add_timestamps(self):
        """Add header editing controls to the window."""
        sorted_timestamps = sorted(self.timestamps, key=lambda x: -x.timestamp)
        for index, timestamp in enumerate(sorted_timestamps):
            self.timestamp_map[index] = timestamp.index
            self.add_timestamp(timestamp.timestamp, index)

    def _add_buttons(self):
        """Add save and cancel buttons."""
        last_row = self.sizer.GetEffectiveRowsCount()
        last_col = self.sizer.GetEffectiveColsCount() - 1

        save_button = wx.Button(self.panel, label=_("Restore"))
        cancel_button = wx.Button(self.panel, label=_("Cancel"))

        self.sizer.Add(save_button, pos=(last_row, last_col - 1), flag=wx.ALIGN_RIGHT,
                       border=self.args.padding)
        self.sizer.Add(cancel_button, pos=(last_row, last_col), flag=wx.ALIGN_RIGHT,
                       border=self.args.padding)

        self.Bind(wx.EVT_BUTTON, self._ok_pressed, save_button)
        self.Bind(wx.EVT_BUTTON, lambda evt: self.EndModal(wx.ID_CANCEL), cancel_button)

    def _ok_pressed(self, _evt):
        if self.list_ctrl.GetFirstSelected() < 0:
            wx.MessageBox(_("No backup selected!"))
        else:
            self.Close()
            index = self.list_ctrl.GetFirstSelected()
            with progress_dialog.Dialog(
                    _("Restore backup"), _("Restoring backup"), style=wx.PD_AUTO_HIDE) as dlg:
                dlg.run_function(
                    self.model.restore_backup, self.timestamp_map[index], initial_msg=_("Restoring backup"))
            self.EndModal(wx.ID_SAVE)
