""" A module for using the application from the GUI.
"""
# -*- coding: utf-8 -*-
from functools import partial
import locale
import logging
import math
import os
import pkg_resources
import sys
import traceback
import wx
from wx import html
from wx import ribbon as RB
from pubsub import pub

from kvalikirstu2 import argument_parser
from kvalikirstu2 import backup_select_window
from kvalikirstu2 import gui_utils
from kvalikirstu2 import gui_model
from kvalikirstu2 import progress_dialog
from kvalikirstu2 import header_panel
from kvalikirstu2 import lang_select_window
from kvalikirstu2 import exceptions
from kvalikirstu2 import text_add_window
from kvalikirstu2 import text_replace_window
from kvalikirstu2 import tmp_edit_window
from kvalikirstu2 import utils
from kvalikirstu2.localization import _

logger = logging.getLogger('kvalikirstu2.gui')


class MainWindow(wx.Frame):
    """The main frame for the program."""
    def __init__(self, model, *args, **kwargs):
        """Inits the main frame."""
        super(MainWindow, self).__init__(*args, **kwargs)
        self.model = model
        self.args = argument_parser.get_args()
        self.exists_listeners = {gui_model.Messages.INDEX_EXISTS: [],
                                 gui_model.Messages.STUDY_EXISTS: [],
                                 gui_model.Messages.TEMP_EXISTS: []}
        self.padding = self.args.padding
        self.index_exists = False
        self.generate_index_id = None
        self.index_panel = None
        self.index_bar = None
        self._init_help()
        self._init_ui()

        pub.subscribe(self._update_lang_menu, gui_model.Messages.LANGUAGES)
        pub.subscribe(self._study_exists_updated, gui_model.Messages.STUDY_EXISTS)
        pub.subscribe(self._index_exists_updated, gui_model.Messages.INDEX_EXISTS)
        pub.subscribe(self._temp_exists_updated, gui_model.Messages.TEMP_EXISTS)
        pub.subscribe(self._update_title, gui_model.Messages.STUDY_EXISTS)
        pub.subscribe(self._update_index_button, gui_model.Messages.INDEX_EXISTS)
        self.model.broadcast_initial_messages()

    def _update_title(self, exists):
        """Updates the title of the window.
        """
        title_with_version = 'Kvalikirstu2 (v.{})'.format(pkg_resources.get_distribution('kvalikirstu2').version)
        if exists:
            metadata = self.model.get_metadata()
            self.SetTitle('%s: %s - %s' % (metadata.study_id, self.model.get_study_path(), title_with_version))
        else:
            self.SetTitle(title_with_version)

    def add_button_exists_hook(self, button_bar, button_id, msg):
        """Enable/disable this button in the button bar based on the message type msg.
        """
        self.exists_listeners[msg].append((button_bar, button_id))

    def _study_exists_updated(self, exists):
        """
        """
        for button_bar, button_id in self.exists_listeners[gui_model.Messages.STUDY_EXISTS]:
            button_bar.EnableButton(button_id, exists)

    def _index_exists_updated(self, exists):
        """
        """
        for button_bar, button_id in self.exists_listeners[gui_model.Messages.INDEX_EXISTS]:
            button_bar.EnableButton(button_id, exists)

    def _temp_exists_updated(self, exists):
        """
        """
        for button_bar, button_id in self.exists_listeners[gui_model.Messages.TEMP_EXISTS]:
            button_bar.EnableButton(button_id, exists)

    def _ask_to_convert(self):
        formats = utils.CONVERTABLE_FORMATS
        msg = _(
            "{}{} files found\n\n"
            "These files must be converted for Kvalikirstu2.\n"
            "Convert files to odt?").format(
                ", ".join(formats[:-1]), _(" or ") + formats[-1] if len(formats) > 1 else "")

        with wx.MessageDialog(self, msg, style=wx.YES_NO, caption=_("Convert?")) as dlg:
            dlg_return = dlg.ShowModal()
            if dlg_return == wx.ID_YES:
                self._convert(_("Convert to ODT"), self.model.convert_to_odt)

    def _init_ui(self):
        """Inits the UI components of the main frame.

        """
        self.panel = wx.Panel(self)
        sizer = wx.BoxSizer(wx.VERTICAL)

        self.index_lang_menu = wx.Menu()

        self.header_panel = header_panel.HeaderPanel(self.panel, self, model=self.model,
                                                     style=wx.TAB_TRAVERSAL | wx.BORDER_THEME)
        self._ribbon = RB.RibbonBar(self.panel, wx.ID_ANY)
        self._init_ribbon_bar()

        sizer.Add(self._ribbon, 0, wx.EXPAND)
        sizer.Add(self.header_panel, 1, wx.EXPAND)

        self.panel.SetSizerAndFit(sizer)
        self.SetTitle("Kvalikirstu 2")

        self.Bind(wx.EVT_CLOSE, lambda evt: self._on_close())
        self.SetBackgroundColour(gui_utils.COLOUR_LIGHT)

        ribbon_w = self._ribbon.GetSize()[0]

        min_width = 320
        min_height = 240
        scale = wx.ScreenDC().GetUserScale()
        self.SetMinSize((min_width, min_height))
        screen_size = wx.DisplaySize()
        screen_size = (screen_size[0], screen_size[1] - 55)  # taskbar size
        max_width = math.ceil(max(min_width, screen_size[0] / scale[0] - self.padding * 2))
        max_height = math.ceil(max(min_height, screen_size[1] / scale[1] - self.padding * 2))
        self.SetMaxSize((max_width, max_height))
        self.SetSize(ribbon_w + self.padding*4, 768)
        self.CenterOnScreen()

    def _open_command_menu(self, evt):
        command_menu = wx.Menu()
        for index, cmd in enumerate(self.model.get_commands()):
            item = command_menu.Append(index, cmd.text, cmd.help_text)
            self.Bind(wx.EVT_MENU, partial(self._execute_command, index), item)
        evt.PopupMenu(command_menu)

    def _init_commands(self, button_bar, btn_id):
        if not self.model.get_commands():
            return

        button_bar.AddButton(btn_id, _("Tools"), gui_utils.create_bitmap("apps"), _("Launch external tools"),
                             kind=RB.RIBBON_BUTTON_DROPDOWN)
        button_bar.Bind(RB.EVT_RIBBONBUTTONBAR_DROPDOWN_CLICKED, self._open_command_menu, id=btn_id)

    def _execute_command(self, index, _evt):
        self.model.execute_command(index)

    def _init_ribbon_bar(self):
        """Inits the main ribbon toolbar.

        """
        source_page = RB.RibbonPage(self._ribbon)
        btn_id = wx.ID_HIGHEST + 1

        self._init_study_panel(source_page, btn_id)
        gui_utils.add_separator_to_ribbon_page(source_page)

        self._init_prep_panel(source_page, btn_id)
        gui_utils.add_separator_to_ribbon_page(source_page)

        self._init_text_panel(source_page, btn_id)
        gui_utils.add_separator_to_ribbon_page(source_page)

        self._init_tools_panel(source_page, btn_id)
        gui_utils.add_separator_to_ribbon_page(source_page)

        self._init_index_panel(source_page, btn_id)
        gui_utils.add_separator_to_ribbon_page(source_page)

        help_bar = gui_utils.new_buttonbar(source_page)
        help_bar.AddButton(btn_id, _("Help"), gui_utils.create_bitmap("help"), _("Open help book"))
        help_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, lambda evt: self._open_help(), id=btn_id)
        btn_id += 1
        self._init_commands(help_bar, btn_id)

        gui_utils.set_ribbon_colours(self._ribbon)
        self._ribbon.Realize()

    def _init_study_panel(self, page, btn_id):
        """Inits the ribbon study panel.

        """
        study_panel = RB.RibbonPanel(page, wx.ID_ANY, _("Select source"), style=RB.RIBBON_PANEL_NO_AUTO_MINIMISE)
        study_bar = RB.RibbonButtonBar(study_panel, wx.ID_ANY)

        study_bar.AddButton(btn_id, _("Select study"), gui_utils.create_bitmap("browse"),
                            _("Open a file browse window for selecting the study to process"))
        study_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, lambda evt: self._select_study(), id=btn_id)
        btn_id += 1

        study_bar.AddButton(btn_id, _("Restore backup"), gui_utils.create_bitmap("backup"),
                            bitmap_disabled=gui_utils.create_bitmap("backup_disabled"),
                            help_string=_("Load study files from backup"))
        study_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, lambda evt: self._open_backup_restore(), id=btn_id)
        self.add_button_exists_hook(study_bar, btn_id, gui_model.Messages.STUDY_EXISTS)

    def _init_prep_panel(self, page, btn_id):
        """Inits the ribbon prep panel.

        """
        prep_panel = RB.RibbonPanel(page, wx.ID_ANY, _("Source files: Format"), style=RB.RIBBON_PANEL_NO_AUTO_MINIMISE)
        prep_bar = RB.RibbonButtonBar(prep_panel, wx.ID_ANY)

        prep_bar.AddButton(btn_id, _("Rename files"), gui_utils.create_bitmap("list_view"),
                           bitmap_disabled=gui_utils.create_bitmap("list_view_disabled"),
                           kind=RB.RIBBON_BUTTON_DROPDOWN)
        prep_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, self._show_rename_menu, id=btn_id)
        prep_bar.Bind(RB.EVT_RIBBONBUTTONBAR_DROPDOWN_CLICKED, self._show_rename_menu, id=btn_id)
        self.add_button_exists_hook(prep_bar, btn_id, gui_model.Messages.STUDY_EXISTS)
        btn_id += 1

        prep_bar.AddButton(btn_id, _("Convert"), gui_utils.create_bitmap("convert"),
                           bitmap_disabled=gui_utils.create_bitmap("convert_disabled"), kind=RB.RIBBON_BUTTON_DROPDOWN)
        prep_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, self._show_conversion_menu, id=btn_id)
        prep_bar.Bind(RB.EVT_RIBBONBUTTONBAR_DROPDOWN_CLICKED, self._show_conversion_menu, id=btn_id)
        self.add_button_exists_hook(prep_bar, btn_id, gui_model.Messages.STUDY_EXISTS)

    def _init_text_panel(self, page, btn_id):
        """Inits the ribbon text panel.

        """
        text_panel = RB.RibbonPanel(page, wx.ID_ANY, _("Source files: Write"), style=RB.RIBBON_PANEL_NO_AUTO_MINIMISE)
        text_bar = RB.RibbonButtonBar(text_panel, wx.ID_ANY)

        text_bar.AddButton(btn_id, _("Add text to data files"), gui_utils.create_bitmap("add_text"),
                           bitmap_disabled=gui_utils.create_bitmap("add_text_disabled"), kind=RB.RIBBON_BUTTON_DROPDOWN,
                           help_string=_("Add citation requirement or free text to data files"))
        text_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, self._show_text_menu, id=btn_id)
        text_bar.Bind(RB.EVT_RIBBONBUTTONBAR_DROPDOWN_CLICKED, self._show_text_menu, id=btn_id)
        self.add_button_exists_hook(text_bar, btn_id, gui_model.Messages.STUDY_EXISTS)
        btn_id += 1

        text_bar.AddButton(btn_id, _("Save"), gui_utils.create_bitmap("save"),
                           bitmap_disabled=gui_utils.create_bitmap("save_disabled"), kind=RB.RIBBON_BUTTON_DROPDOWN,
                           help_string=_("Save headers"))
        text_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, self._show_save_menu, id=btn_id)
        text_bar.Bind(RB.EVT_RIBBONBUTTONBAR_DROPDOWN_CLICKED, self._show_save_menu, id=btn_id)
        self.add_button_exists_hook(text_bar, btn_id, gui_model.Messages.STUDY_EXISTS)

    def _init_tools_panel(self, page, btn_id):
        """Inits the ribbon tools panel.

        """
        tools_panel = RB.RibbonPanel(page, wx.ID_ANY, _("Tools"), style=RB.RIBBON_PANEL_NO_AUTO_MINIMISE)
        tools_bar = RB.RibbonButtonBar(tools_panel, wx.ID_ANY)

        tools_bar.AddButton(btn_id, _("Create header list"), gui_utils.create_bitmap("new"),
                            bitmap_disabled=gui_utils.create_bitmap("new_disabled"),
                            help_string=_("Create a header list for a study with non-text files (pdf, jpg...)"))
        tools_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, lambda evt: self._init_external_study(), id=btn_id)
        self.add_button_exists_hook(tools_bar, btn_id, gui_model.Messages.STUDY_EXISTS)
        btn_id += 1

        tools_bar.AddButton(btn_id, _("Delete temp files"), gui_utils.create_bitmap("delete"),
                            bitmap_disabled=gui_utils.create_bitmap("delete_disabled"),
                            help_string=_("Delete the Temp folder in the study directory."))
        tools_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, self.delete_tempfiles, id=btn_id)
        self.add_button_exists_hook(tools_bar, btn_id, gui_model.Messages.TEMP_EXISTS)
        btn_id += 1

        if utils.is_windows():
            tools_bar.AddButton(btn_id, _("Open study folder"), gui_utils.create_bitmap("open_folder"),
                                bitmap_disabled=gui_utils.create_bitmap("open_folder_disabled"),
                                help_string=_("Opens the folder with the current study."))
            tools_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED,
                           lambda evt: self.model.open_data_folder(),
                           id=btn_id)
            self.add_button_exists_hook(tools_bar, btn_id, gui_model.Messages.STUDY_EXISTS)

    def _init_index_panel(self, page, btn_id):
        """Inits the ribbon index panel.

        """
        index_panel = RB.RibbonPanel(page, wx.ID_ANY, _("Index"), style=RB.RIBBON_PANEL_NO_AUTO_MINIMISE)
        index_bar = RB.RibbonButtonBar(index_panel, wx.ID_ANY)
        self.index_panel = index_panel
        self.index_bar = index_bar

        index_bar.AddButton(btn_id, _("Index language"),
                            gui_utils.create_bitmap("lang"),
                            bitmap_disabled=gui_utils.create_bitmap("lang_disabled"),
                            kind=RB.RIBBON_BUTTON_DROPDOWN,
                            help_string=_("Set the language of the index file (select before generating index)."))
        self.add_button_exists_hook(index_bar, btn_id, gui_model.Messages.STUDY_EXISTS)
        index_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED,
                       lambda evt: evt.PopupMenu(self.index_lang_menu), id=btn_id)
        index_bar.Bind(RB.EVT_RIBBONBUTTONBAR_DROPDOWN_CLICKED,
                       lambda evt: evt.PopupMenu(self.index_lang_menu), id=btn_id)
        btn_id += 1

        index_bar.AddButton(btn_id, _("Open index"), gui_utils.create_bitmap("open_index"),
                            bitmap_disabled=gui_utils.create_bitmap("open_index_disabled"),
                            help_string=_("Open the generated HTML index."))
        index_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED, lambda evt: self.open_index(), id=btn_id)
        self.add_button_exists_hook(index_bar, btn_id, gui_model.Messages.INDEX_EXISTS)
        btn_id += 1

        self.generate_index_id = btn_id
        index_bar.AddButton(self.generate_index_id, _("Generate index"), gui_utils.create_bitmap("run"),
                            bitmap_disabled=gui_utils.create_bitmap("run_disabled"),
                            help_string=_("Generate a new index or updates existing index with current headers."))
        self.add_button_exists_hook(index_bar, btn_id, gui_model.Messages.STUDY_EXISTS)
        self.index_bar.Bind(RB.EVT_RIBBONBUTTONBAR_CLICKED,
                            lambda evt: self.generate_index(), id=btn_id)

    def _update_index_button(self, exists):
        if exists == self.index_exists:
            return
        self.index_bar.DeleteButton(self.generate_index_id)
        self.index_exists = exists

        if exists:
            text = _("Update index")
        else:
            text = _("Generate index")
        self.index_bar.AddButton(self.generate_index_id, text, gui_utils.create_bitmap("run"),
                                 bitmap_disabled=gui_utils.create_bitmap("run_disabled"),
                                 help_string=_("Generate a new index or updates existing index with current headers."))
        self.index_bar.Realize()

    def _update_lang_menu(self, languages, selected_language):
        for language, display_name in languages:
            item = self.index_lang_menu.AppendRadioItem(wx.ID_ANY, display_name)

            def change_language(lang, _evt):
                self.model.select_index_language(lang)

            bound = partial(change_language, language)
            self.Bind(wx.EVT_MENU, bound, item)
            item.Check(selected_language == language)

    def _show_text_menu(self, evt):
        menu = wx.Menu()
        self.Bind(wx.EVT_MENU, self.write_citreq,
                  menu.Append(-1, _("Add citation requirement"),
                              _("Add citation requirement text to each data file in the study")))
        self.Bind(wx.EVT_MENU, lambda evt: self._open_text_add_window(),
                  menu.Append(-1, _("Add free text"), _("Add text to the beginning of each data file in the study")))
        self.Bind(wx.EVT_MENU, lambda evt: self._open_text_replace_window(),
                  menu.Append(-1, _("Replace text"), _("Replace existing text")))
        evt.PopupMenu(menu)

    def _show_rename_menu(self, evt):
        menu = wx.Menu()
        self.Bind(
            wx.EVT_MENU,
            lambda evt: self.model.rename_daf_files(),
            menu.Append(-1, _("Rename files to 'daFxxxx'"), _("Rename all data files to a uniform format")))

        self.Bind(
            wx.EVT_MENU,
            lambda evt: self._open_lang_select_window(),
            menu.Append(-1, _("Rename with lang codes"), _("Add language code to all data file names")))

        evt.PopupMenu(menu)

    def _show_conversion_menu(self, evt):
        menu = wx.Menu()

        encoding_title = _("TXT encoding")
        self.Bind(wx.EVT_MENU,
                  lambda evt: self._convert(encoding_title, self.model.convert_txt_encoding),
                  menu.Append(-1, encoding_title, _("Convert TXT encoding to UTF-8")))

        txt_title = _("→ TXT/UTF-8")
        self.Bind(wx.EVT_MENU,
                  lambda evt: self._convert(txt_title, self.model.convert_to_txt),
                  menu.Append(-1, txt_title))

        odt_title = _("→ ODT/UTF-8")
        self.Bind(wx.EVT_MENU,
                  lambda evt: self._convert(odt_title, self.model.convert_to_odt),
                  menu.Append(-1, odt_title))

        evt.PopupMenu(menu)

    def _set_size(self, size, _evt):
        self.model.set_table_width(size)

    def _show_save_menu(self, evt):
        menu = wx.Menu()
        self.Bind(wx.EVT_MENU,
                  lambda evt: self.model.save_changes(),
                  menu.Append(-1, _("Save changes"),
                              _("Save changes to headers.")))
        self.Bind(wx.EVT_MENU,
                  lambda evt: self.model.save_changes_to_daf(),
                  menu.Append(-1, _("Save changes to daF"),
                              _("Save changes into data files.")))
        evt.PopupMenu(menu)

    def _open_lang_select_window(self):
        with lang_select_window.LangSelectWindow(self, self.model) as win:
            win.ShowModal()

    def _init_external_study(self):
        """Inits a study with external files.

        """
        msg = "" if not self.model.daf_container_exists() else _("Header list already exists. Overwrite header list?")
        callback = (lambda lines: self.model.setup_external_file_study([l for l in lines.split('\n') if l]))

        with text_add_window.TextAddWindow(_("Write headers for study"), callback, msg) as dialog:
            dialog.ShowModal()

    def generate_index(self):
        """Generates the index file with current settings.

        """
        with progress_dialog.Dialog(
                title=_("Generate index for study"), message="", parent=self, style=wx.PD_APP_MODAL) as dlg:
            tempfiles_generated = self.generate_tempfiles(dlg)

            if tempfiles_generated:
                dlg.run_function(self.model.generate_output, initial_msg=_("Generating index..."))

    def generate_tempfiles(self, dlg):
        """Generates tempfiles for the index. Runs in a loop so that the user
        can resolve any error that comes up.

        """
        while True:
            try:

                dlg.run_function(self.model.generate_tempfiles, initial_msg=("Generating tempfiles..."))
                return True
            except exceptions.TempFileException as exception:
                inner_exc = exception.inner_exception
                if isinstance(inner_exc, exceptions.ParagraphParsingException):
                    return_val = _display_error_message(inner_exc, str(exception), True, self.model)
                    if return_val != wx.ID_SAVE:
                        wx.MessageBox(_("Process was aborted."), _("Run aborted"), wx.OK)
                        return False
                else:
                    raise exception

    def check_unsaved_changes(self, msg=None, yes_label=None, no_label=None):
        """Checks if there are unsaved changes and shows a message dialog if found.
        Returns False if cancel is pressed, otherwise returns True.

        """
        if not msg:
            msg = _("There are unsaved changes.\nSave changes?")
        if not yes_label:
            yes_label = _("Save")
        if not no_label:
            no_label = _("Discard")

        if self.model.check_unsaved_changes():
            with wx.MessageDialog(self, msg, style=wx.YES_NO | wx.CANCEL) as dlg:
                dlg.SetYesNoLabels(yes_label, no_label)
                dlg_return = dlg.ShowModal()
                if dlg_return == wx.ID_YES:
                    self.model.save_changes()
                return dlg_return
        return wx.ID_NO

    def delete_tempfiles(self, _evt):
        """Delete temporary files and show success message if user confirms the action. """
        msg = _("Delete all temporary files?")
        yes_label = _("Delete")
        no_label = _("Cancel")
        with wx.MessageDialog(self, msg, style=wx.YES_NO) as dlg:
            dlg.SetYesNoLabels(yes_label, no_label)
            dlg_return = dlg.ShowModal()
            if dlg_return == wx.ID_YES:
                self.model.delete_tempfiles()

    def write_citreq(self, _evt):
        """Write citation requirements and show success message if user confirms the action. """
        msg = _("Write citation requirement text to all data files?")
        yes_label = _("Write")
        no_label = _("Cancel")
        with wx.MessageDialog(self, msg, style=wx.YES_NO) as dlg:
            dlg.SetYesNoLabels(yes_label, no_label)
            dlg_return = dlg.ShowModal()
            if dlg_return != wx.ID_YES:
                return
        with progress_dialog.Dialog(
                title=_("Citation requirements"),
                message=_("Writing citation requirements to all text files..."),
                parent=self
                ) as dlg:
            dlg.run_function(
                self.model.generate_citation_requirement, initial_msg=_("Generating citation requirements"))

    def open_index(self):
        """Opens index file if generated.
        """
        self.model.open_index()

    def _convert(self, title, conversion_func):
        """Opens a progress dialog and starts converting files.
        """
        with progress_dialog.Dialog(title=title, message="", parent=self, style=wx.PD_APP_MODAL) as dlg:
            dlg.run_function(conversion_func, initial_msg=_("Preparing conversion"))

    def _open_text_add_window(self):
        """Opens the text adding window.

        """
        with text_add_window.TextAddWindow(_("Add text to daFs"), self.model.add_text_to_dafs) as dialog:
            dialog.ShowModal()

    def _on_close(self):
        check = self.check_unsaved_changes(msg=_("There are unsaved changes made after previous index creation.\n"
                                                 "Save changes before exit?"),
                                           yes_label=_("Save changes"), no_label=_("Discard changes"))
        if check != wx.ID_CANCEL:
            self.Destroy()

    def _select_study(self):
        """Opens the directory selection dialog and sets the selected study path."""
        check = self.check_unsaved_changes(msg=_("There are unsaved changes made after previous index creation.\n"
                                                 "Save changes before selecting a new study?"),
                                           yes_label=_("Save changes"), no_label=_("Discard changes"))
        if check == wx.ID_CANCEL:
            return

        with wx.DirDialog(
                None, _("Select study directory"), "", wx.DD_DEFAULT_STYLE | wx.DD_DIR_MUST_EXIST) as dlg:
            if dlg.ShowModal() == wx.ID_CANCEL:
                return
            path = dlg.GetPath()

        with progress_dialog.Dialog(_("Read study"), _("Reading study files..."), parent=self) as dlg:
            convert = dlg.run_function(self.model.set_study_path, path, initial_msg=_("Parsing study files..."))

        if convert:
            self._ask_to_convert()

    def _init_help(self):
        self.html_window = html.HtmlHelpController(self, style=wx.html.HF_TOOLBAR |
                                                   wx.html.HF_CONTENTS | wx.html.HF_SEARCH)
        self.html_window.AddBook(os.path.join(os.path.dirname(__file__), "help", "help_fin", "help.hhp"))
        self.html_window.AddBook(os.path.join(os.path.dirname(__file__), "help", "help_eng", "help.hhp"))

    def _open_help(self):
        self.html_window.DisplayContents()
        pos = self.GetPosition()
        self.html_window.GetFrame().SetSize((1200, 768))
        self.html_window.GetFrame().Move((pos[0] + 20, pos[1] + 20))
        self.html_window.GetFrame().Refresh()

    def _open_backup_restore(self):
        """Opens backup restoring window."""
        with backup_select_window.BackupSelectWindow(self.model) as dialog:
            dialog.ShowModal()

    def _open_text_replace_window(self):
        """Opens the text replacing window.
        """
        with text_replace_window.TextReplaceWindow(self.model) as dialog:
            dialog.ShowModal()


def _show_message(msg):
    """Displays a message box with the given message.
    """
    wx.MessageBox(msg, "Info")


def _load_locale():
    """ Loads the locale for the GUI application.

    """
    loc_id = locale.getdefaultlocale()
    locale_info = wx.Locale.FindLanguageInfo(loc_id[0])
    # wxPython oddity: Initializing this instance sets the app locale
    # Additionally it has to be assigned, otherwise the locale is not set
    _loc = wx.Locale(locale_info.Language)
    return _loc


def _exception_hook(etype, value, trace, model):
    """
    Exception hook for catching GUI exceptions.

    :param etype: Exception type.
    :param str value: The message.
    :param traceback trace: The traceback for the exception.

    """
    trace_info = ''.join(traceback.format_exception(etype, value, trace))
    logger.error("Exception: %s", trace_info)

    _display_error_message(None, "%s : %s" % (_("Error"), value), False, model)


def _display_error_message(exception, message, temp_error, model):
    """Displays an error dialog with a message.

    :param exception: The exception that was raised.
    :param str message: The message to be displayed.
    :param bool temp_error: Did the error occur during generating temp files.

    """
    if temp_error:
        message = (_("There was an error generating temp files. Go edit the corresponding .temp file based on the"
                     " error.") + "\n\n" + message)
        if isinstance(exception, exceptions.ParagraphParsingException):
            return tmp_edit_window.open_temp_error(model, message, exception)
    return wx.MessageBox(message, _("Error"), wx.OK | wx.ICON_ERROR)


def main():
    """Runs the main window loop.

    """
    settings = argument_parser.Settings()
    settings.set_up_gui_settings(sys.argv[1:])
    model = gui_model.GUIModel()
    sys.excepthook = partial(_exception_hook, model=model)
    app = wx.App(False)
    window = MainWindow(model, None)
    window.Show()

    _load_locale()
    app.MainLoop()


if __name__ == "__main__":
    main()
