"""A module for a custom progress dialog with threading support.
"""

import concurrent.futures
import time
from pubsub import pub
import wx
from kvalikirstu2 import gui_model
from kvalikirstu2.localization import _


class Dialog(wx.GenericProgressDialog):
    """A progress dialog with threading support and progress event handling.
    """
    def __init__(self, title, message, parent=None, style=wx.PD_AUTO_HIDE | wx.PD_APP_MODAL):
        super().__init__(title=title, message=message, parent=parent, style=style)
        pub.subscribe(self._update, gui_model.Messages.TASK_PROGRESS)
        pub.subscribe(self._finish, gui_model.Messages.TASK_FINISHED)
        self._current_msg = ""
        self._finished_msg = ""

    def __exit__(self, exc_type, exc_val, exc_tb):
        pub.unsubscribe(self._update, gui_model.Messages.TASK_PROGRESS)
        pub.unsubscribe(self._finish, gui_model.Messages.TASK_FINISHED)
        super().__exit__(exc_type, exc_val, exc_tb)

    def _update(self, msg):
        self._current_msg = msg

    def _finish(self, msg):
        self._finished_msg = msg

    def run_function(self, function, *args, initial_msg=""):
        """Run function in a separate thread while updating the progress bar.

        :param function: Function to be run.
        :param initial_msg: Initial message to be displayed above the progress bar.
        :return: The return value of the function being ran.

        """
        with concurrent.futures.ThreadPoolExecutor(max_workers=1) as executor:
            future = executor.submit(function, *args)
            self._current_msg = initial_msg

            while future.running():
                if self._current_msg:
                    self.Pulse(self._current_msg)
                time.sleep(0.01)

            result = future.result()
            if self._finished_msg:
                box_title = _("Info")
                self.Hide()
                wx.MessageBox(self._finished_msg, box_title)
            return result
