""" Tests the argument parser module.
"""
import os
import unittest
import sys
import tempfile
from unittest.mock import patch
from kvalikirstu2 import exceptions
from kvalikirstu2 import main
from kvalikirstu2 import argument_parser
from kvalikirstu2 import arg_parser
from tests import test_writer
from tests import test_kvali_interface


class ArgumentParserTest(unittest.TestCase, test_writer.CitReqMixin, test_kvali_interface.TestDataVerifierMixin):
    def test_parse_args(self):
        parser = arg_parser.Kvalikirstu2ConfigArgParser()
        parser.add_argument('--a', type=str)

        parser.parse_args(['--a', 'b'])
        with self.assertRaises(arg_parser.ConfigFileParsingException):
            parser.parse_args(['--b', 'b'])

    def test_selected_headers(self):
        testargs1 = ['', '--parse', '--id', 'test', '--path', os.path.join('tests', 'test files', 'valid'),
                     '--headers', 'NUMERO', 'SUKUPUOLI', 'KAUPUNKI', '--overwrite-temp']
        testargs2 = ['', '--parse', '--id', 'test', '--path', os.path.join('tests', 'test files', 'valid'),
                     '--headers', '', '--overwrite-temp']
        with patch.object(sys, 'argv', testargs1):
            main.main()

        with self.assertRaises(exceptions.TempFileException):
            with patch.object(sys, 'argv', testargs2):
                main.main()

    def test_integration_main(self):

        testargs1 = ['', '--parse', '--id', 'test', '--path', os.path.join('tests', 'test files', 'valid'),
                     '--overwrite-temp']
        testargs2 = ['', '--generate', '--id', 'test', '--path', os.path.join('tests', 'test files', 'valid'),
                     '--overwrite-temp', '--output', os.path.join('tests', 'test files', 'valid')]
        testargs3 = ['', '--citreq', '--id', 'test', '--path', self.get_citreq_dir(),
                     '--overwrite-temp']

        with patch.object(sys, 'argv', testargs1):
            main.main()

        with patch.object(sys, 'argv', testargs2):
            main.main()

        self.populate_citreq()

        with patch.object(sys, 'argv', testargs3):
            main.main()

        self.unpopulate_citreq()

    def test_parse_none_args(self):
        testargs1 = ['', '--index-lang', 'test']
        with patch.object(sys, 'argv', testargs1):
            argument_parser.Settings().set_up_cli_settings(None)
            args = argument_parser.get_args()
            self.assertEqual('test', args.index_lang)

    def test_convert(self):
        with tempfile.TemporaryDirectory() as tempdir:
            testargs = ['', 'test', '--path', tempdir, '--convert', 'encoding']
            test_kvali_interface.populate_test_folder(tempdir, "qwertyuiopåöä", 10, 'cp1252')

            with patch.object(sys, 'argv', testargs):
                main.main()

    @patch('kvalikirstu2.kvali_interface.rename_headers_in_data_files')
    def test_header_rename(self, mock):
        with tempfile.TemporaryDirectory() as tempdir:
            testargs = ['', '--path', tempdir, '--rename-headers']
            test_kvali_interface.populate_test_folder(tempdir, "qwertyuiopåöä", 10, 'cp1252')

            with patch.object(sys, 'argv', testargs):
                main.main()

            self.assertTrue(mock.called)

    @unittest.skipIf(os.getenv('NO_WX'), 'Skipping tests that require wxPython')
    def test_handler_for_gui(self):
        settings = argument_parser.Settings()
        settings.set_up_gui_settings([])
        self.assertIn('gui', settings.log_config['loggers']['kvalikirstu2']['handlers'])
        self.assertNotIn('console', settings.log_config['loggers']['kvalikirstu2']['handlers'])

    def test_handler_for_cli(self):
        settings = argument_parser.Settings()
        settings.set_up_cli_settings([])
        self.assertNotIn('gui', settings.log_config['loggers']['kvalikirstu2']['handlers'])
        self.assertIn('console', settings.log_config['loggers']['kvalikirstu2']['handlers'])

    def test_external_data_study(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            directory = os.path.join(temp_dir, "study_path")
            self.setup_folder_as_ext_study_folder(directory)
            testargs1 = ['', '--setup_external_file_study', '--path', directory,
                         '--headers', 'Description', 'test']

            with patch.object(sys, 'argv', testargs1):
                main.main()
            self.verify_external_study_data(['Description', 'test'])

    def test_register_extra_interface_raises(self):
        with self.assertRaises(AssertionError):
            argument_parser.register_archive_interface(None)

    def test_language_codes(self):
        argument_parser.Settings().set_up_cli_settings([])
        expected_codes = ['fin', 'eng', 'swe', 'zxx']
        codes = argument_parser.get_languages()
        for code in expected_codes:
            self.assertIn(code, codes)

    @patch('kvalikirstu2.kvali_interface.restore_backup')
    @patch('kvalikirstu2.kvali_interface.set_study_path')
    def test_restore_backup(self, set_mock, restore_mock):
        args = ['', '--restore', '1']
        with patch.object(sys, 'argv', args):
            main.main()
        restore_mock.assert_called_with(1)
        set_mock.assert_called_with(None)

    @patch('kvalikirstu2.kvali_interface.replace_text')
    @patch('kvalikirstu2.kvali_interface.set_study_path')
    def test_replace_text(self, set_mock, replace_mock):
        args = ['', '--replace', '1', '2']
        with patch.object(sys, 'argv', args):
            main.main()
        replace_mock.assert_called_with('1', '2')
        set_mock.assert_called_with(None)
