"""Tests for the convert_encoding module."""

import os
import shutil
import tempfile
import unittest
import subprocess
from kvalikirstu2 import argument_parser
from kvalikirstu2 import converter
from kvalikirstu2 import utils
from kvalikirstu2 import kvali_interface
from tests import test_writer


class TestConvertEncoding(unittest.TestCase):
    def setUp(self):
        test_files_path = os.path.join(os.path.dirname(__file__), "test files", "convertable files")
        argument_parser.Settings().parse_gui(['--max-backup-folder-size', '0'])
        self.temp_dir = tempfile.TemporaryDirectory()
        self.temp_path = os.path.join(self.temp_dir.name, "test")
        shutil.copytree(test_files_path, self.temp_path)

    def encoding_str_test(self, text, encoding):
        encoded = text.encode(encoding, errors='ignore')
        content = converter.convert(encoded)
        self.assertEqual(text, content)

    def create_file_in_folder(self, temp_dir, text, encoding):
        filepath = utils.get_temp_path(temp_dir, ".txt")
        with open(filepath, "w", encoding=encoding) as stream:
            stream.write(text)
        return filepath

    def file_content_test(self, filepath, text):
        with open(filepath, "r", encoding='utf-8-sig') as stream:
            lines = stream.read()

        self.assertEqual(text, lines)

    def read_content_from_file(self, filepath):
        content = kvali_interface.get_textfile_content(filepath)
        return content

    def encoding_text_test(self, text, encoding):
        with tempfile.TemporaryDirectory() as temp_dir:
            filepath = self.create_file_in_folder(temp_dir, text, encoding)
            converter.convert_file(filepath, 'utf-8-sig')
            self.file_content_test(filepath, text)

    def test_convert_ascii(self):
        self.encoding_str_test("qwertyuiopasdfghjklzxcvbnm", 'ascii')

    def test_convert_cp1252(self):
        self.encoding_str_test("qwertyuiopasdfghjklzxcvbnmåäö", 'cp1252')

    def test_convert_utf8(self):
        self.encoding_str_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8')

    def test_convert_utf8sig(self):
        self.encoding_str_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8-sig')

    def test_file_conversion(self):
        self.encoding_text_test("qwertyuiopasdfghjklzxcvbnm", 'ascii')

    def test_file_conversion_cp1252(self):
        self.encoding_text_test("qwertyuiopasdfghjklzxcvbnmåäö", 'cp1252')

    def test_file_conversion_utf8(self):
        self.encoding_text_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8')

    def test_file_conversion_utf8sig(self):
        self.encoding_text_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8-sig')

    def folder_conversion_test(self, text, encoding, file_count):
        with tempfile.TemporaryDirectory() as temp_dir:
            files = []

            for _ in range(file_count):
                path = self.create_file_in_folder(temp_dir, text, encoding)
                files.append(path)

            yield from converter.convert_file_encoding_in_folder(temp_dir, 'utf-8-sig', '.txt')

            for file in files:
                self.file_content_test(file, text)

    def subfolder_conversion_test(self, text, encoding, file_count):
        with tempfile.TemporaryDirectory() as temp_dir:
            files = []
            converted_files = []

            for _ in range(file_count):
                path = self.create_file_in_folder(temp_dir, text, encoding)
                files.append(path)

            os.makedirs(os.path.join(temp_dir, 'sub'))
            sub_path = os.path.join(temp_dir, 'sub')

            for _ in range(file_count):
                path = self.create_file_in_folder(sub_path, text, encoding)
                files.append(path)

            for file_path in converter.convert_file_encoding_in_folder(temp_dir, 'utf-8-sig', '.txt'):
                converted_files.append(file_path)

            for file in files:
                self.file_content_test(file, text)
                self.assertTrue(file in converted_files)

    def test_folder_conversion(self):
        self.folder_conversion_test("qwertyuiopasdfghjklzxcvbnm", 'ascii', 10)
        self.folder_conversion_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8-sig', 10)
        self.folder_conversion_test("qwertyuiopasdfghjklzxcvbnmåäö", 'cp1252', 10)
        self.folder_conversion_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8', 10)

    def test_subfolder_conversion(self):
        self.subfolder_conversion_test("qwertyuiopasdfghjklzxcvbnm", 'ascii', 10)
        self.subfolder_conversion_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8-sig', 10)
        self.subfolder_conversion_test("qwertyuiopasdfghjklzxcvbnmåäö", 'cp1252', 10)
        self.subfolder_conversion_test("qwertyuiopasdfghjklzxcvbnmåäö", 'utf-8', 10)

    @unittest.skipIf(os.getenv('NO_ODT'), 'Skipping tests that require LibreOffice')
    def test_libreoffice_conversion_to_odt(self):
        docx_path = os.path.join(self.temp_path, "data", "test_data1.docx")
        data_path = os.path.join(self.temp_path, "data")
        odt_path = os.path.join(data_path, "test_data1.odt")

        self.assertFalse(os.path.isfile(odt_path))
        for f in converter.convert_files_with_libreoffice_to_odt(data_path, 30):
            self.assertTrue(os.path.isfile(odt_path))

        content = self.read_content_from_file(odt_path)
        expected_content = self.read_content_from_file(os.path.join(self.temp_path, "test_data1.txt"))
        self.assertFalse(os.path.exists(docx_path))
        self.assertEqual(expected_content, content)

    @unittest.skipIf(os.getenv('NO_ODT'), 'Skipping tests that require LibreOffice')
    def test_libreoffice_conversion_to_txt(self):
        docx_path = os.path.join(self.temp_path, "data", "test_data1.docx")
        data_path = os.path.join(self.temp_path, "data")
        txt_path = os.path.join(data_path, "test_data1.txt")

        self.assertFalse(os.path.isfile(txt_path))
        for f in converter.convert_files_with_libreoffice_to_txt(data_path, 30):
            self.assertTrue(os.path.isfile(txt_path))
        converter.convert_file(txt_path, 'utf-8')

        lines1 = test_writer.read_lines_from_file(txt_path)
        lines2 = test_writer.read_lines_from_file(os.path.join(self.temp_path, "test_data1.txt"))
        self.assertFalse(os.path.exists(docx_path))
        self.assertEqual(lines1, lines2)

    @unittest.skipIf(os.getenv('NO_ODT'), 'Skipping tests that require LibreOffice')
    def test_timeout_low_throws(self):
        data_path = os.path.join(self.temp_path, "data")
        with self.assertRaises(subprocess.TimeoutExpired):
            for f in converter.convert_files_with_libreoffice_to_txt(data_path, 0.001):
                pass
