import os
import time
import tempfile
import unittest
from kvalikirstu2 import folder_backup


def create_file(filepath, char_count=1):
    with open(filepath, 'w') as fp:
        fp.write('a' * char_count)


class TestFolderBackup(unittest.TestCase):
    """Unit tests for the folder_backup module."""

    def setup_directory(self, temp_dir):
        self.backup_directory = os.path.join(temp_dir, "backup")
        os.mkdir(self.backup_directory)
        folder_backup.clear_backups(self.backup_directory)

    def tearDown(self):
        folder_backup.clear_backups(self.backup_directory)

    def test_folder_backup(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            self.setup_directory(temp_dir)

            file_a = os.path.join(self.backup_directory, 'a.txt')
            file_b = os.path.join(self.backup_directory, 'b.txt')
            create_file(file_a)
            folder_backup.backup_folder(self.backup_directory, 1e6, 15)
            create_file(file_b)

            os.unlink(file_a)
            os.unlink(file_b)
            folder_backup.restore_backup(self.backup_directory, 0)
            self.assertTrue(os.path.exists(file_a))
            self.assertFalse(os.path.exists(file_b))

    def test_folder_backup_rotation(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            self.setup_directory(temp_dir)

            for i in range(6):
                create_file(os.path.join(self.backup_directory, 'a.txt'), i+1)
                folder_backup.backup_folder(self.backup_directory, 1e6, 5)
                time.sleep(0.1)
            self.assertEqual(1, folder_backup._get_next_index(self.backup_directory,
                                                              os.path.join(self.backup_directory, "backup"), 5))
            self.assertEqual(5, len(folder_backup.get_timestamps(self.backup_directory, 5)))

    def test_too_big_folder(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            self.setup_directory(temp_dir)
            file = os.path.join(self.backup_directory, 'a.txt')
            create_file(file, 2)
            folder_backup.backup_folder(self.backup_directory, 1, 5)
            self.assertEqual(0, len(folder_backup.get_timestamps(self.backup_directory, 5)))

    def test_folder_backup_clear(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            self.setup_directory(temp_dir)
            file = os.path.join(self.backup_directory, 'a.txt')
            create_file(file)
            folder_backup.backup_folder(self.backup_directory, 1, 5)
            folder_backup.clear_backups(self.backup_directory)
            self.assertEqual(0, len(folder_backup.get_timestamps(self.backup_directory, 5)))
