""" Tests for gui classes.
"""
# Skip this module if NO_WX in env
import pytest
import os
if os.getenv('NO_WX'):
    pytest.skip('Skipping module because wxPython is not available', allow_module_level=True)

import os
import unittest
import wx
from copy import deepcopy
from pubsub import pub
from unittest import mock
from kvalikirstu2 import argument_parser
from kvalikirstu2 import exceptions
from kvalikirstu2 import gui
from kvalikirstu2 import gui_model
from kvalikirstu2 import header_panel
from kvalikirstu2 import kvali_interface
from kvalikirstu2 import paragraph_analyzer
from kvalikirstu2 import utils
from kvalikirstu2 import writer
from kvalikirstu2 import tmp_edit_window
from kvalikirstu2 import study
from tests import test_writer
from kvalikirstu2 import text_add_window
from kvalikirstu2 import text_replace_window
from kvalikirstu2 import progress_dialog

test_files_path = os.path.join(os.path.dirname(__file__), "test files")


@mock.patch('kvalikirstu2.gui.MainWindow._ask_to_convert', mock.Mock())
@mock.patch('wx.MessageBox', mock.Mock(return_value=wx.ID_OK))
@mock.patch('wx.Dialog.ShowModal', mock.Mock(return_value=wx.ID_CANCEL))
class TestGUI(unittest.TestCase):
    def setUp(self):
        self.app = wx.App(False)
        argument_parser.Settings().parse_gui(['--max-backup-folder-size', '0'])
        pub.unsubAll()
        model = gui_model.GUIModel()
        self.window = gui.MainWindow(model, None)
        self.header_panel = self.window.header_panel

    def set_study_path(self):
        self.header_panel.model.set_study_path(test_files_path)

    def set_ext_study_path(self):
        self.header_panel.model.set_study_path(os.path.join(test_files_path, "external file test"))

    @mock.patch('kvalikirstu2.progress_dialog.Dialog')
    def test_generate_index(self, _mock):
        self.set_study_path()
        self.window.generate_index()

    @mock.patch('kvalikirstu2.progress_dialog.Dialog')
    def test_convert(self, _mock):
        self.set_study_path()
        self.window._convert("converting", self.window.model.convert_to_txt)
        self.window._convert("converting", self.window.model.convert_to_odt)
        self.window._convert("converting", self.window.model.convert_txt_encoding)

    @mock.patch('kvalikirstu2.progress_dialog.Dialog')
    @mock.patch('wx.DirDialog.ShowModal', mock.MagicMock(return_value=wx.ID_OK))
    @mock.patch('wx.DirDialog.GetPath', mock.MagicMock(return_value=test_files_path))
    def test_browse(self, _mock):
        self.window._select_study()
        try:
            kvali_interface.get_temp_glob_pattern()
        except kvali_interface.InvalidProgramStateException:
            self.fail("Study path was not set!")

    def test_init_header_panel(self):
        self.assertEqual(self.header_panel.list_ctrl.GetItemCount(), 0)
        self.assertEqual(self.header_panel.list_ctrl.GetColumnCount(), 4)

    def test_add_files(self):
        self.assertEqual(self.header_panel.list_ctrl.GetItemCount(), 0)
        self.set_study_path()
        self.assertEqual(self.header_panel.list_ctrl.GetItemCount(), len(self.header_panel.model.visible_headers))

    def test_set_header_status(self):
        self.set_study_path()
        header = self.header_panel.model.visible_headers[0]
        self.header_panel.list_ctrl.Select(0)
        self.header_panel.set_header_status(gui_model.HeaderStatus.HIDE)
        header = self.header_panel.model.visible_headers[0]
        self.assertEqual(header.status, gui_model.HeaderStatus.HIDE)

    def test_set_multiple_header_status(self):
        self.set_study_path()
        self.header_panel.model.set_type_shown(gui_model.HeaderType.NOT_HEADER, True)
        self.header_panel.model.set_type_shown(gui_model.HeaderType.BUILTIN, True)
        self.header_panel.list_ctrl.Select(1)
        self.header_panel.list_ctrl.Select(2)

        self.header_panel.set_header_status(gui_model.HeaderStatus.NOT_HEADER)
        self.assertEqual(self.header_panel.model.visible_headers[1].status, gui_model.HeaderStatus.NOT_HEADER)
        self.assertEqual(self.header_panel.model.visible_headers[2].status, gui_model.HeaderStatus.NOT_HEADER)

    def test_rename_header(self):
        self.set_study_path()
        self.header_panel.list_ctrl.editor.SetValue("new header")
        self.header_panel.list_ctrl.curRow = 0
        self.header_panel._remap_edited_header()
        header = self.header_panel.model.visible_headers[0]
        self.assertEqual(header.edited_header, 'new header')

    def test_align_selected(self):
        self.set_study_path()
        header_0 = self.header_panel.model.visible_headers[0]
        header_1 = self.header_panel.model.visible_headers[1]
        self.assertEqual(header_0.alignment, study.Align.LEFT)
        self.assertEqual(header_1.alignment, study.Align.LEFT)
        self.header_panel.list_ctrl.Select(1)
        self.header_panel.align_selected(study.Align.RIGHT)
        header_0 = self.header_panel.model.visible_headers[0]
        header_1 = self.header_panel.model.visible_headers[1]
        self.assertEqual(header_0.alignment, study.Align.LEFT)
        self.assertEqual(header_1.alignment, study.Align.RIGHT)

    @mock.patch('kvalikirstu2.kvali_interface.save_headers')
    @mock.patch('kvalikirstu2.kvali_interface.rename_headers_in_data_files')
    def test_save_changes(self, _mock, __mock):
        self.set_study_path()
        header_0 = self.header_panel.model.visible_headers[0]
        header_1 = self.header_panel.model.visible_headers[1]
        header_2 = self.header_panel.model.visible_headers[2]

        self.header_panel.list_ctrl.editor.SetValue("new header")
        self.header_panel.list_ctrl.curRow = 0
        self.header_panel._remap_edited_header()

        self.header_panel.list_ctrl.Select(1)
        self.header_panel.align_selected(study.Align.RIGHT)
        self.header_panel.list_ctrl.Select(1, 0)

        self.header_panel.list_ctrl.Select(2)
        self.header_panel.set_header_status(gui_model.HeaderStatus.HIDE)
        self.header_panel.list_ctrl.Select(2, 0)
        self.header_panel.set_header_status(gui_model.HeaderStatus.NOT_HEADER)

        header_0 = self.header_panel.model.visible_headers[0]
        header_1 = self.header_panel.model.visible_headers[1]
        header_2 = self.header_panel.model.visible_headers[2]

        self.assertEqual(header_0.edited_header, 'new header')
        self.assertEqual(header_1.alignment, study.Align.RIGHT)
        self.assertEqual(header_2.status, gui_model.HeaderStatus.HIDE)

    @mock.patch('wx.Dialog.EndModal')
    @mock.patch('wx.Dialog.ShowModal')
    def test_value_edit_saves_new_value_to_headerline(self, _mock, __mock):
        self.set_study_path()
        rows = self.header_panel.model.visible_headers
        header_line = rows[0]
        copy_header_line = deepcopy(header_line)
        self.header_panel.list_ctrl.Select(0)

        self.assertEqual(self.header_panel.list_ctrl.GetItemText(0), header_line.original_header)
        edit_vals_dlg = header_panel.ValueEditDlg(
                            self.header_panel, copy_header_line.get_values(), copy_header_line.original_header)

        key = copy_header_line.original_header
        pair = [(key, value) for orig_val, value in edit_vals_dlg.values.items()][0]
        edit_vals_dlg.list_ctrl.Select(0)
        self.assertEqual(edit_vals_dlg.list_ctrl.GetItemText(0), pair[1])
        edit_vals_dlg.list_ctrl.editor.SetValue("TEST_VAL")
        edit_vals_dlg.remap_edited_values()

        self.assertNotEqual(rows[0]._values[pair], "TEST_VAL")

        new_header_values = edit_vals_dlg.get_updated_values_map()
        self.header_panel._change_header_property(
            lambda header: header.update_changed_values(new_header_values))
        self.assertEqual(rows[0].get_values()[pair], "TEST_VAL")

    @mock.patch('wx.Dialog.EndModal')
    @mock.patch('wx.Dialog.ShowModal')
    def test_multiple_value_edit(self, _mock, __mock):
        self.set_study_path()
        rows = self.header_panel.model.visible_headers
        header_line = rows[0]
        copy_header_line = deepcopy(header_line)
        self.header_panel.list_ctrl.Select(0)

        self.assertEqual(self.header_panel.list_ctrl.GetItemText(0), header_line.original_header)
        edit_vals_dlg = header_panel.ValueEditDlg(
                            self.header_panel, copy_header_line.get_values(), copy_header_line.original_header)

        key = copy_header_line.original_header
        pairs = [(key, value) for orig_val, value in edit_vals_dlg.values.items()]
        pair_1 = pairs[0]
        pair_2 = pairs[1]

        edit_vals_dlg.list_ctrl.Select(0)
        edit_vals_dlg.list_ctrl.Select(1)
        self.assertEqual(edit_vals_dlg.list_ctrl.GetItemText(0), pair_1[1])
        self.assertEqual(edit_vals_dlg.list_ctrl.GetItemText(1), pair_2[1])

        edit_vals_dlg.list_ctrl.editor.SetValue("TEST_VAL")
        edit_vals_dlg.remap_edited_values()

        self.assertNotEqual(rows[0]._values[pair_1], "TEST_VAL")
        self.assertNotEqual(rows[0]._values[pair_2], "TEST_VAL")

        new_header_values = edit_vals_dlg.get_updated_values_map()
        self.header_panel._change_header_property(
            lambda header: header.update_changed_values(new_header_values))

        self.assertEqual(rows[0]._values[pair_1], "TEST_VAL")
        self.assertEqual(rows[0]._values[pair_2], "TEST_VAL")

    @mock.patch('wx.Dialog.EndModal')
    @mock.patch('wx.Dialog.ShowModal', mock.MagicMock(return_value=wx.ID_OK))
    def test_open_edit_value_dlg(self, _mock_endmodal):
        self.set_study_path()
        self.header_panel.open_edit_value_dlg()

    def test_redo(self):
        self.set_study_path()

        self.header_panel.list_ctrl.Select(2)
        self.header_panel.align_selected(study.Align.CENTER)
        self.header_panel.list_ctrl.Select(2, 0)

        self.assertEqual(self.header_panel.model.visible_headers[2].alignment, study.Align.CENTER)
        self.header_panel.model.undo()
        self.assertEqual(self.header_panel.model.visible_headers[2].alignment, study.Align.LEFT)

        headers = self.header_panel.model.visible_headers
        self.header_panel.list_ctrl.Select(0)
        self.header_panel.set_header_status(gui_model.HeaderStatus.HIDE)
        self.assertEqual(headers[0].status, gui_model.HeaderStatus.HIDE)

        self.assertEqual(headers[0].status, gui_model.HeaderStatus.HIDE)
        self.header_panel.model.undo()
        self.assertEqual(headers[0].status, gui_model.HeaderStatus.SHOW)
        self.header_panel.model.redo()
        self.assertEqual(headers[0].status, gui_model.HeaderStatus.HIDE)

    @mock.patch('kvalikirstu2.kvali_interface.save_headers')
    @mock.patch('kvalikirstu2.kvali_interface.rename_headers_in_data_files')
    @mock.patch('wx.MessageDialog.ShowModal', mock.MagicMock(return_value=wx.ID_YES))
    def test_check_unsaved_changes_yes_saves(self, _mock, __mock):
        self.set_study_path()
        header = self.header_panel.list_ctrl.GetItemText(1)
        new_align = study.Align.CENTER

        self.assertNotEqual(kvali_interface.get_header_info().header_alignment[header], new_align)
        self.header_panel.list_ctrl.Select(1)
        self.header_panel.align_selected(new_align)
        self.assertTrue(self.window.check_unsaved_changes())

        self.assertEqual(kvali_interface.get_header_info().header_alignment[header], new_align)

    @mock.patch('kvalikirstu2.kvali_interface.generate_tempfiles', mock.MagicMock())
    @mock.patch('wx.GenericProgressDialog.__init__', mock.MagicMock())
    def test_generate_tempfiles_no_exception_handled(self):
        self.set_study_path()
        dlg = progress_dialog.Dialog(title="", message="")
        self.assertTrue(self.window.generate_tempfiles(dlg))
        self.app.Destroy()

    @mock.patch('kvalikirstu2.kvali_interface.generate_tempfiles',
                mock.MagicMock(side_effect=exceptions.
                               TempFileException(paragraph_analyzer.
                                                 ParagraphParsingException(test_files_path, 0, 1, ""))))
    @mock.patch('kvalikirstu2.kvali_interface.delete_tempfiles', mock.MagicMock())
    @mock.patch('wx.GenericProgressDialog.__init__', mock.MagicMock())
    def test_generate_tempfiles_paragraph_exception_handling_canceled(self):
        self.set_study_path()
        dlg = progress_dialog.Dialog(title="", message="")
        self.assertFalse(self.window.generate_tempfiles(dlg))
        self.app.Destroy()

    @mock.patch('kvalikirstu2.kvali_interface.generate_tempfiles', mock.MagicMock(side_effect=Exception()))
    @mock.patch('wx.GenericProgressDialog.__init__', mock.MagicMock())
    def test_generate_tempfiles_other_than_paragraph_exception_raised(self):
        self.set_study_path()
        dlg = progress_dialog.Dialog(title="", message="")
        with self.assertRaises(Exception):
            self.window.generate_tempfiles(dlg)
            self.app.Destroy()

    def test_cant_change_builtin_to_daf(self):
        header_info = study.HeaderInfo()
        header_info.init_builtin_headers()
        line = gui_model.HeaderLine('HTML', header_info)
        with self.assertRaises(gui_model.StatusError):
            line.set_status(gui_model.HeaderStatus.DAF_HEADER)


class TestGuiWindows(unittest.TestCase, test_writer.CitReqMixin):
    def setUp(self):
        self.app = wx.App()
        argument_parser.Settings().parse_gui(['--max-backup-folder-size', '0'])

    def test_tmp_edit_window(self):
        model = gui_model.GUIModel()
        temp_file = utils.get_temp_path(test_files_path, ".temp")
        writer.write_txt(temp_file, "test")
        exception = paragraph_analyzer.ParagraphParsingException(temp_file, 0, 1)
        tmp_edit_window.TmpEditWindow(model, exception, None)
        tmp_edit_window.ParagraphErrorWindow(model, "", exception)
        os.unlink(temp_file)

    def test_text_add_window(self):
        text_add_window.TextAddWindow('test', None)

    @mock.patch('wx.Dialog.EndModal')
    def test_window_callback(self, _mck):
        cb_mock = mock.MagicMock()
        window = text_add_window.TextAddWindow('test', cb_mock)
        window._done_pressed(None)
        cb_mock.assert_called_with('')

    def test_text_replace_window(self):
        text_replace_window.TextReplaceWindow(None)