""" Tests the template generator.
"""

import unittest
import os
import shutil
import tempfile
from unittest import mock
from kvalikirstu2 import argument_parser
from kvalikirstu2 import exceptions
from kvalikirstu2 import header_scanner
from kvalikirstu2 import kvali_interface
from kvalikirstu2 import utils
from kvalikirstu2.data_archive_interface import DummyInterface
from kvalikirstu2 import localization
from tests import test_writer


def get_empty_mockdata():
    return {'study_path': None,
            'study_title': None,
            'data_path': None,
            'study_id': None,
            'current_study': None,
            'cit_req': None,
            'index_path': None,
            'header_info': None,
            'interface': DummyInterface()}


def populate_test_folder(temp_dir, text, file_count, encoding, random_names=True):
    files = []
    data_path = os.path.join(temp_dir, "data")
    os.mkdir(data_path)

    for index in range(file_count):
        if random_names:
            filepath = utils.get_temp_path(data_path, ".txt")
        else:
            filepath = os.path.join(data_path, str(index) + ".txt")
        with open(filepath, "w", encoding=encoding) as stream:
            stream.write(text)
        files.append(filepath)
    return files


class TestDataVerifierMixin:
    def verify_external_study_data(self, headers):
        args = argument_parser.get_args()
        study_path = kvali_interface._DATA['study_path']
        kvali_interface.set_study_path(study_path)
        kvali_interface.generate_tempfiles()
        header_info = kvali_interface._DATA['header_info']
        current_study = kvali_interface.get_current_study()
        _ = localization.get_translation_func(args.index_lang)
        header_info.daf_header = _('Data file')

        for subject in current_study.subjects:
            subject.resolve_daf(header_info)
            print(subject.data_file)

        self.assertEqual(3, len(current_study.subjects))
        for header in headers:
            self.assertEqual('', current_study.subjects[0].headers[header])
            self.assertEqual('', current_study.subjects[1].headers[header])

        self.assertEqual(os.path.join('data', 'daFxxxx.txt'), os.path.relpath(current_study.subjects[0].data_file,
                                                                              study_path))
        self.assertEqual(os.path.join('data', 'daFxxxx_1.pdf'), os.path.relpath(current_study.subjects[1].data_file,
                                                                                study_path))
        self.assertEqual(os.path.join('data', 'daFxxxx_2.txt'), os.path.relpath(current_study.subjects[2].data_file,
                                                                                study_path))

        temp_path = os.path.join(study_path, "temp", "data-daFxxxx_2.txt.temp")
        temp_lines = test_writer.read_lines_from_file(temp_path)
        self.assertEqual("**BEGINDATA**", temp_lines[0])
        self.assertEqual("Text1: a", temp_lines[1])
        self.assertEqual("Text2: b", temp_lines[2])
        self.assertEqual("Text3: c", temp_lines[3])
        self.assertEqual("**ENDDATA**", temp_lines[4])

    def setup_folder_as_ext_study_folder(self, folder):
        shutil.copytree(os.path.join(os.path.dirname(__file__), "test files", "external file test"), folder)


class TestKvaliInterface(unittest.TestCase, TestDataVerifierMixin):
    def setUp(self):
        argument_parser.Settings().set_up_cli_settings(['--max-backup-folder-size', '0'])

    def get_testdir(self):
        return os.path.join(os.path.dirname(__file__), "test files")

    def get_file_count(self, glob_str):
        files = utils.natsorted_glob(glob_str)
        print(files)
        return len(files)

    def test_tempfile_writing(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_path = utils.get_temp_path(temp_dir, ".txt")
            kvali_interface.save_tempfile_content(temp_path, "Test")
            self.assertEqual("Test%s" % os.linesep, kvali_interface.get_textfile_content(temp_path))

    def test_temp_files_location(self):
        testdir = self.get_testdir()
        kvali_interface.set_study_path(testdir)
        kvali_interface.delete_tempfiles()

        temp_files_glob = os.path.join(testdir, "temp", "**")
        data_files_glob = os.path.join(testdir, "data", "**")

        start_count = self.get_file_count(data_files_glob)
        print("test temp files")
        self.assertEqual(0, self.get_file_count(temp_files_glob))
        kvali_interface.generate_tempfiles()

        self.assertEqual(start_count, self.get_file_count(data_files_glob))
        self.assertGreater(self.get_file_count(temp_files_glob), 0)

    def test_valid_inputs(self):
        testdir = self.get_testdir()
        kvali_interface.set_study_path(testdir)
        kvali_interface.generate_tempfiles()
        assert kvali_interface.temp_files_exist()
        kvali_interface.generate_output()
        assert kvali_interface.index_exists()

        index_path = kvali_interface.get_index_path()
        temp_path = kvali_interface.get_study_json_path()
        header_path = kvali_interface.get_headers_json_path()

        kvali_interface.delete_tempfiles()
        os.unlink(index_path)
        self.assertFalse(os.path.isfile(temp_path))
        self.assertFalse(os.path.isfile(header_path))
        self.assertFalse(kvali_interface.temp_files_exist())

    def test_interface_remap(self):
        testdir = self.get_testdir()
        prior_header = "AMMATTI"
        post_header = "PROFESSION"
        deleted_header_index = 4

        header_info = header_scanner.get_header_info_for_study(os.path.join(testdir, "data"))
        kvali_interface.set_study_path(testdir)
        kvali_interface.generate_tempfiles()
        lstudy = kvali_interface.get_current_study()
        self.assertEqual(lstudy.subjects[0].headers[prior_header], "Hissinapin painaja")

        headers = header_info.header_mapping
        headers[prior_header] = post_header
        headers_removed = []
        headers_removed.append("TESTI ATTR")

        kvali_interface.set_headers(headers, headers_removed)
        lstudy = kvali_interface.get_current_study()
        lstudy.remap_headers(header_info)
        self.assertEqual(lstudy.subjects[0].headers[post_header], "Hissinapin painaja")

        with self.assertRaises(Exception):
            header_info.headers[deleted_header_index]
            print(header_info.headers[deleted_header_index])

    def test_header_index_map(self):
        testdir = self.get_testdir()
        kvali_interface.set_study_path(testdir)
        kvali_interface.generate_tempfiles()
        header_info = kvali_interface.get_header_info()

        self.assertEqual(3, header_info.header_indexes["AMMATTI"])
        self.assertEqual(6, header_info.header_indexes["daF"])
        self.assertEqual(-1, header_info.header_indexes["HTML"])

        indexes = dict(header_info.header_indexes)
        kvali_interface.set_headers(header_info.header_mapping,
                                    [], None)
        self.assertEqual(3, header_info.header_indexes["AMMATTI"])
        self.assertEqual(6, header_info.header_indexes["daF"])
        self.assertEqual(-1, header_info.header_indexes["HTML"])

        indexes["HTML"] = 6
        indexes["daF"] = 7
        kvali_interface.set_headers(header_info.header_mapping,
                                    [], indexes)

        self.assertEqual(6, header_info.header_indexes["HTML"])
        self.assertEqual(7, header_info.header_indexes["daF"])

    def test_interface_remap_to_same(self):
        """Load a study and map all the headers to one header. Should result in one header."""
        testdir = self.get_testdir()

        kvali_interface.set_study_path(testdir)
        kvali_interface.generate_tempfiles()
        header_info = kvali_interface.get_header_info()

        headers = {}
        for key in header_info.header_mapping:
            headers[key] = "test"

        kvali_interface.set_headers(headers, [])
        lstudy = kvali_interface.get_current_study()
        lstudy.remap_headers(header_info)
        print(header_info.get_index_headers())
        self.assertEqual(len(header_info.get_index_headers()), 6)  # 1 header + built-in

    def test_invalid_header_set(self):
        """Tries to set invalid headers and asserts a syntax error is raised"""
        testdir = self.get_testdir()

        kvali_interface.set_study_path(testdir)
        kvali_interface.generate_tempfiles()
        header_info = kvali_interface.get_header_info()

        headers = {}
        for key in header_info.header_mapping:
            headers[key] = ""

        with self.assertRaises(SyntaxError):
            kvali_interface.set_headers(headers, [])

    def test_nostudy_returns_false(self):
        functions = [kvali_interface.study_exists,
                     kvali_interface.temp_files_exist,
                     kvali_interface.paragraph_files_exist,
                     kvali_interface.index_exists]

        with mock.patch.dict(kvali_interface._DATA, get_empty_mockdata()):
            for function in functions:
                self.assertFalse(function())

    def test_nostudy_raises_calls(self):
        functions = [kvali_interface.get_temp_glob_pattern,
                     kvali_interface.get_headers_json_path,
                     kvali_interface.get_study_json_path,
                     kvali_interface.generate_tempfiles,
                     kvali_interface.generate_output,
                     kvali_interface.generate_citreq,
                     kvali_interface.delete_tempfiles,
                     kvali_interface.get_current_study,
                     kvali_interface.get_selected_headers]

        with mock.patch.dict(kvali_interface._DATA, get_empty_mockdata()):
            with self.assertRaises(kvali_interface.InvalidProgramStateException):
                kvali_interface.set_headers(None, None)

            for function in functions:
                with self.assertRaises(kvali_interface.InvalidProgramStateException):
                    function()
            with self.assertRaises(kvali_interface.InvalidProgramStateException):
                for test in kvali_interface.convert_to_txt():
                    pass

    def test_selected_headers(self):
        testdir = self.get_testdir()
        kvali_interface.set_study_path(testdir)
        headers = kvali_interface.get_selected_headers()
        headers['AMMATTI'] = False
        kvali_interface.set_selected_headers(headers)
        self.assertFalse(kvali_interface.get_selected_headers()['AMMATTI'])

    def convert_data_encoding_test(self, text, encoding):
        with tempfile.TemporaryDirectory() as temp:
            files = populate_test_folder(temp, text, 1, encoding)
            kvali_interface.set_study_path(temp)
            yield from kvali_interface.convert_to_txt()
            with open(files[0], "r", encoding='utf-8-sig') as stream:
                output_text = stream.read()

            self.assertEqual(text, output_text)

    def test_convert_data_encoding(self):
        text = ("FSDxxxx Otsikko. Aineistoon ja sen tekijöihin tulee viitata asianmukaisesti kaikissa julkaisuissa"
                " ja esityksissä, joissa aineistoa käytetään. Tietoarkiston antaman malliviittaustiedon voi merkitä"
                " lähdeluetteloon sellaisenaan tai sitä voi muokata julkaisun käytäntöjen"
                " mukaisesti.")

        self.convert_data_encoding_test(text, 'cp1252')
        self.convert_data_encoding_test(text, 'utf-8')
        self.convert_data_encoding_test(text, 'utf-8-sig')

    def test_external_file_conversion(self):
        argument_parser.Settings().set_up_cli_settings(['--max-backup-folder-size', '0'])
        with tempfile.TemporaryDirectory() as temp_dir:
            directory = os.path.join(temp_dir, "study_path")
            self.setup_folder_as_ext_study_folder(directory)
            with mock.patch.dict(kvali_interface._DATA, get_empty_mockdata()):
                kvali_interface.set_study_path(directory)
                kvali_interface.setup_external_file_study(['Description', 'test1'])
                self.verify_external_study_data(['Description', 'test1'])
                index_headers = kvali_interface._DATA['header_info'].header_indexes
                self.assertEqual(0, index_headers[localization._("Data file")])
                self.assertEqual(1, index_headers["Description"])
                self.assertEqual(2, index_headers["test1"])
                self.assertEqual(3, index_headers["Folder"])

    def test_header_selection_serialization(self):
        with tempfile.TemporaryDirectory() as temp:
            populate_test_folder(temp, "text", 1, "utf-8")
            kvali_interface.set_study_path(temp)
            kvali_interface.set_selected_headers({'header': True})
            self.assertTrue(kvali_interface.get_selected_headers()['header'])

            with mock.patch.dict(kvali_interface._DATA, get_empty_mockdata()):
                kvali_interface.set_study_path(temp)
                self.assertTrue(kvali_interface.get_selected_headers()['header'])

    def test_get_files(self):
        mock_data = {'data_path': os.path.join(os.path.dirname(__file__), "test files", "get_files_test", "data"),
                     'study_path': os.path.join(os.path.dirname(__file__), "test files", "get_files_test")}
        with mock.patch.dict(kvali_interface._DATA, mock_data):
            files = kvali_interface.get_all_files_from_data_folder()
            relative_files = set([os.path.relpath(path, mock_data['data_path']) for path in files])
            self.assertEqual(set(['test.pdf', 'test.txt', os.path.join('folder', 'test.pdf')]), relative_files)

    @mock.patch('kvalikirstu2.writer.replace_in_folder')
    def test_replace_text(self, mock_function):
        with tempfile.TemporaryDirectory() as temp_dir:
            mock_data = {'study_path': temp_dir, 'data_path': os.path.join(temp_dir, 'data')}
            with mock.patch.dict(kvali_interface._DATA, mock_data):
                kvali_interface.replace_text('a', 'b')
                mock_function.assert_called_with(mock_data['data_path'], 'a', 'b')

    def test_add_language_codes(self):
        with tempfile.TemporaryDirectory() as temp:
            kvali_interface.set_study_path(temp)
            populate_test_folder(temp, "text", 1, "utf-8", False)
            path = os.path.join(temp, "data", "0.txt")
            kvali_interface.add_language_codes({path: 'fin'})
            self.assertTrue(os.path.exists(os.path.join(temp, "data", "0_fin.txt")))
            self.assertFalse(os.path.exists(path))

    def test_get_daf_data(self):
        with tempfile.TemporaryDirectory() as temp:
            kvali_interface.set_study_path(temp)
            populate_test_folder(temp, "text", 1, "utf-8", False)
            path = os.path.join(temp, "data", "0.txt")
            data = kvali_interface.get_daf_data()
            self.assertEqual(kvali_interface.DaFData(path, '', 'text'), data[0])

    def test_add_language_code_to_existing_code(self):
        with tempfile.TemporaryDirectory() as temp:
            kvali_interface.set_study_path(temp)
            populate_test_folder(temp, "text", 1, "utf-8", False)
            path = os.path.join(temp, "data", "0.txt")
            kvali_interface.add_language_codes({path: 'fin'})
            self.assertTrue(os.path.exists(os.path.join(temp, "data", "0_fin.txt")))
            self.assertFalse(os.path.exists(path))

    def test_pdf_path(self):
        path = os.path.join(os.path.dirname(__file__), 'test files', 'pdf')
        kvali_interface.set_study_path(path)
        self.assertTrue(kvali_interface.study_exists())

    def test_invalid_headers_for_daf_container(self):
        with self.assertRaises(exceptions.InvalidHeaderException):
            kvali_interface._write_daf_container_for_external_study(None, ['a:', 'b:'])

    def test_header_list_doesnt_add_header_list(self):
        with tempfile.TemporaryDirectory() as temp:
            kvali_interface.set_study_path(temp)
            populate_test_folder(temp, "text", 1, "utf-8", False)
            path = os.path.join(temp, "data", "0.txt")
            header_list_path = kvali_interface.get_daf_container_path()
            kvali_interface._write_daf_container_for_external_study(header_list_path, [])

            output_text = ""
            with open(header_list_path, mode="r", encoding="utf-8") as stream:
                output_text = stream.read()
            kvali_interface._write_daf_container_for_external_study(header_list_path, [])

            with open(header_list_path, mode="r", encoding="utf-8") as stream:
                new_text = stream.read()
                self.assertEqual(new_text, output_text)


class InterfaceCitreqTest(unittest.TestCase, test_writer.CitReqMixin):
    def test_citreq(self):
        self.populate_citreq()
        kvali_interface.set_study_path(self.get_citreq_dir())
        kvali_interface.generate_citreq()

        for path in utils.get_supported_files(self.get_citreq_datadir()):
            lines = self.read_lines_from_file(path)
            self.assertEqual(lines, ["citreq"])
        self.unpopulate_citreq()
