""" Tests the paragraph reader.
"""
import os
import io
import pytest
import tempfile
import time
import unittest
from unittest import mock
from kvalikirstu2 import paragraph_analyzer
from kvalikirstu2 import argument_parser
from tests import test_writer


@pytest.fixture(name='testdir')
def get_testdir():
    return os.path.join(os.path.dirname(__file__), "test files", "valid", "data")


@pytest.fixture(name='settings')
def set_up_settings_class():
    return argument_parser.Settings()


@pytest.fixture(name='store_args')
def store_test_args(settings, testdir):
    settings.parse_command_line([None, '--path', testdir, '--parse', '--overwrite_temp'])
    argument_parser.store_settings(settings.settings)


class TestParagraph():
    @pytest.fixture(autouse=True)
    def init_paragraph(self, store_args):
        store_args
        self.paragraph = paragraph_analyzer.Paragraph("")

    def test_init_no_type(self):
        assert self.paragraph.par_type == paragraph_analyzer.ParagraphType.UNKNOWN

    def test_comparison_with_other_type(self):
        assert self.paragraph != 1

    def test_init_with_type(self):
        paragraph_with_type = paragraph_analyzer.Paragraph(None, paragraph_analyzer.ParagraphType.DATA)
        assert paragraph_with_type.par_type == paragraph_analyzer.ParagraphType.DATA

    def test_is_empty_true_when_new_paragraph_no_type(self):
        assert self.paragraph.is_empty()

    def test_paragraph_not_empty_when_has_token(self):
        paragraph_with_type = paragraph_analyzer.Paragraph(None, paragraph_analyzer.ParagraphType.BEGINDATA)
        assert not paragraph_with_type.is_empty()

    def test_is_empty_false_when_lines(self):
        self.paragraph.lines = ["line"]
        assert not self.paragraph.is_empty()

    def test_is_header(self):
        assert not self.paragraph.is_header()
        self.paragraph.par_type = paragraph_analyzer.ParagraphType.HEADER
        assert self.paragraph.is_header()

    def test_set_type(self):
        self.paragraph.set_type(paragraph_analyzer.ParagraphType.HEADER)
        assert self.paragraph.par_type == paragraph_analyzer.ParagraphType.HEADER

    def test_write_to_stream_token(self):
        self.paragraph.set_type(paragraph_analyzer.ParagraphType.BEGINDATA)
        self.paragraph.lines = ["line1", "line2", "line3"]
        test_stream = io.StringIO()
        self.paragraph.write_to_stream(test_stream)
        test_stream.seek(0)
        assert ["**BEGINDATA**" + os.linesep] == test_stream.readlines()

    def test_write_to_stream_no_token(self):
        self.paragraph.set_type(paragraph_analyzer.ParagraphType.HEADER)
        self.paragraph.lines = ["line1", "line2", "line3"]
        test_stream = io.StringIO()
        self.paragraph.write_to_stream(test_stream)
        test_stream.seek(0)
        assert ["line1" + os.linesep, "line2" + os.linesep, "line3" + os.linesep] == test_stream.readlines()

    @mock.patch("kvalikirstu2.utils.is_header_line")
    def test_resolve_type_sets_type_data_when_more_than_min_lines(self, mock_is_header):
        mock_is_header.return_value = True
        self.paragraph.set_type(paragraph_analyzer.ParagraphType.UNKNOWN)
        self.paragraph.lines = ["line1", "line2", "line3"]
        self.paragraph.resolve_type()
        assert self.paragraph.par_type == paragraph_analyzer.ParagraphType.HEADER

    def test_resolve_type_sets_type_data_when_less_than_min_consecutive_headerlike_lines(self):
        self.paragraph.set_type(paragraph_analyzer.ParagraphType.UNKNOWN)
        self.paragraph.lines = ["line1", "line2", "line3", "line: 4"]
        self.paragraph.resolve_type()
        assert self.paragraph.par_type == paragraph_analyzer.ParagraphType.DATA


class TestParagraphReader(unittest.TestCase):

    def paragraphs_valid(self, paragraphs):
        """Check if paragraphs were parsed correctly for a certain document

        :param paragraphs: A list of paragraphs.
        """
        assert (paragraphs[0].lines == ['NUMERO: 111', 'SUKUPUOLI: Muu', 'KAUPUNKI: Tampere',
                                        'AMMATTI: Hissinapin painaja'])
        assert paragraphs[0].is_header()
        assert not paragraphs[1].is_header()
        assert not paragraphs[2].is_header()
        assert not paragraphs[3].is_header()
        assert not paragraphs[4].is_header()
        assert not paragraphs[5].is_header()
        assert not paragraphs[6].is_header()
        assert not paragraphs[7].is_header()
        assert paragraphs[8].is_header()
        assert not paragraphs[9].is_header()
        assert not paragraphs[10].is_header()
        assert not paragraphs[11].is_header()
        assert not paragraphs[12].is_header()
        assert not paragraphs[13].is_header()
        assert not paragraphs[14].is_header()
        assert not paragraphs[15].is_header()
        assert paragraphs[16].is_header()
        assert not paragraphs[17].is_header()
        assert not paragraphs[18].is_header()
        assert not paragraphs[19].is_header()

    @pytest.fixture(autouse=True)
    def init_test(self, testdir, store_args):
        self.testdir = testdir

    def test_txt(self):
        """Tests reading a .txt file"""
        filename = os.path.join(self.testdir, "test_data.txt")
        reader = paragraph_analyzer.ParagraphReader(filename)
        paragraphs = reader.read_paragraphs()
        self.paragraphs_valid(paragraphs)

    def test_odt(self):
        """Test reading an .odt file"""
        filename = os.path.join(self.testdir, "test_data.odt")
        reader = paragraph_analyzer.ParagraphReader(filename)
        paragraphs = reader.read_paragraphs()
        self.paragraphs_valid(paragraphs)

    def par_assert_rows(self, par, min, max):
        self.assertEqual(par.row_min, min)
        self.assertEqual(par.row_max, max)

    def check_paragraph_rows(self, paragraphs):
        self.par_assert_rows(paragraphs[0], 0, 1)
        self.par_assert_rows(paragraphs[1], 2, 3)
        self.par_assert_rows(paragraphs[2], 4, 5)
        self.par_assert_rows(paragraphs[3], 6, 7)
        self.par_assert_rows(paragraphs[4], 8, 10)
        self.par_assert_rows(paragraphs[5], 11, 12)

    def check_paragraph_file(self, path, overwrite_temp):
        reader = paragraph_analyzer.ParagraphReader(path, overwrite_temp)
        paragraphs = reader.read_paragraphs()
        self.check_paragraph_rows(paragraphs)

    def test_row_number_reading(self):
        path = os.path.join(os.path.dirname(__file__), "test files", "paragraph_test", "paragraphs.txt")
        path1 = os.path.join(os.path.dirname(__file__), "test files", "paragraph_test", "paragraphs.odt")
        self.check_paragraph_file(path, True)
        self.check_paragraph_file(path, False)
        self.check_paragraph_file(path1, True)
        self.check_paragraph_file(path1, False)

    def test_paragraph_reading2(self):
        path1 = os.path.join(os.path.dirname(__file__), "test files", "paragraph_test", "paragraphs_2.odt")
        lines = test_writer.read_lines_from_file(path1, False)
        self.assertEqual('Testi1', lines[2])
        self.assertEqual('Testi2', lines[3])
        self.assertEqual('Testi3', lines[4])

    def test_doesnt_ignore_changes_to_original_file(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            with open(os.path.join(temp_dir, "test.txt"), mode='w', encoding='utf-8') as file:
                file.write('test1:\n')
            reader = paragraph_analyzer.ParagraphReader(os.path.join(temp_dir, "test.txt"), study_path=temp_dir, overwrite_temp=False, use_temp=True)
            paragraphs = reader.read_paragraphs()
            lines1 = test_writer.read_lines_from_file(os.path.join(temp_dir, "temp", "test.txt.temp"))
            self.assertEqual('test1:', lines1[0])
            time.sleep(0.1)
            with open(os.path.join(temp_dir, "test.txt"), mode='w', encoding='utf-8') as file:
                file.write('test2:\n')
            reader = paragraph_analyzer.ParagraphReader(os.path.join(temp_dir, "test.txt"), study_path=temp_dir, overwrite_temp=False, use_temp=True)
            paragraphs = reader.read_paragraphs()
            lines2 = test_writer.read_lines_from_file(os.path.join(temp_dir, "temp", "test.txt.temp"))

            self.assertEqual('test2:', lines2[0])

    def test_duplicate_headers_throws(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            path = os.path.join(temp_dir, "file.txt")
            with open(path, mode='w', encoding='utf-8') as file:
                file.write('otsake1:\n')
                file.write('otsake2:\n')
                file.write('otsake1:\n')
                file.write('\n')
            reader = paragraph_analyzer.ParagraphReader(path, False)
            with self.assertRaises(paragraph_analyzer.ParagraphParsingException):
                reader.read_subjects()
