""" Tests the paragraph reader.
"""
import os
import pytest
import unittest
from tempfile import TemporaryDirectory
from kvalikirstu2 import argument_parser
from kvalikirstu2 import odt_reader
from kvalikirstu2 import study
from kvalikirstu2 import study_reader
from kvalikirstu2 import data_file_tempwriter
from kvalikirstu2 import utils


@pytest.fixture(name='testdir')
def get_testdir():
    return os.path.join(os.path.dirname(__file__), "test files", "valid", "data")


@pytest.fixture(name='settings')
def set_up_settings_class():
    return argument_parser.Settings()


@pytest.fixture(name='test_id')
def get_test_id():
    return 'test_id'


@pytest.fixture(name='store_args')
def store_test_args(settings, testdir):
    settings.parse_command_line([None, '--path', testdir, '--parse', '--overwrite_temp'])
    argument_parser.store_settings(settings.settings)


class TestStudyReader(unittest.TestCase):
    """Tests the study reader"""

    @pytest.fixture(autouse=True)
    def init_reader_valid(self, testdir, test_id, store_args):
        store_args
        self.reader = study_reader.StudyReader(test_id, "", study_path=testdir)

    def get_path_all(self):
        return os.path.join(os.path.dirname(__file__), "test files")

    def test_reader_with_valid(self):
        self.reader.get_study()

    def test_get_files_empty_raises_invalid_path_exception(self):
        with TemporaryDirectory() as temp_dir:
            test_reader = study_reader.StudyReader("id_123", "", False, study_path=temp_dir)
            with pytest.raises(utils.InvalidPathException):
                test_reader.get_files()

    def test_get_files_multiple(self):
        with TemporaryDirectory() as temp_dir:
            for i in range(5):
                temp_file = open(os.path.join(temp_dir, 'file' + str(i) + '.txt'), 'w')
                temp_file.write('writing' + str(i))
                temp_file.close()
            test_reader = study_reader.StudyReader("id_123", "", False, study_path=temp_dir)
            self.assertEqual(len(test_reader.get_files()), 5)

    def test_get_files_non_txt_raises_invalid_path_exception(self):
        with TemporaryDirectory() as temp_dir:
            for i in range(5):
                temp_file = open(os.path.join(temp_dir, 'file' + str(i) + '.dat'), 'w')
                temp_file.write('writing' + str(i))
                temp_file.close()
            test_reader = study_reader.StudyReader("id_123", "", False, study_path=temp_dir)
            with pytest.raises(utils.InvalidPathException):
                test_reader.get_files()

    def test_get_files_multiple_ext_only_valid_fetched(self):
        with TemporaryDirectory() as temp_dir:
            for i in range(5):
                temp_file = open(os.path.join(temp_dir, 'file' + str(i) + '.txt'), 'w')
                temp_file.write('writing' + str(i))
                temp_file.close()
            for i in range(5):
                temp_file = open(os.path.join(temp_dir, 'file' + str(i) + '.dat'), 'w')
                temp_file.write('writing' + str(i))
                temp_file.close()
            test_reader = study_reader.StudyReader("id_123", "", False, study_path=temp_dir)
            assert len(test_reader.get_files()), 5

    def test_get_files_multiple_ext(self):
        with TemporaryDirectory() as temp_dir:
            for i in range(5):
                temp_file = open(os.path.join(temp_dir, 'file' + str(i) + '.txt'), 'w')
                temp_file.write('writing' + str(i))
                temp_file.close()
            for i in range(5):
                temp_file = open(os.path.join(temp_dir, 'file' + str(i) + '.dat'), 'w')
                temp_file.write('writing' + str(i))
                temp_file.close()
            test_reader = study_reader.StudyReader("id_123", "", False, study_path=temp_dir)
            print(test_reader.get_files())
            self.assertEqual(len(test_reader.get_files()), 5)

    def test_respect_use_temp_argument(self):
        """Tests that the use_temp argument is properly used."""

        path = os.path.join(self.reader.study_path, "test_data.txt")
        temp_path = data_file_tempwriter.generate_temp_path(path, None)

        # Write something else than what is in the actual file
        with open(temp_path, 'w') as fp:
            fp.write("1:2\n2:3\n3:4\nnot a header line")

        # Use temp file
        temp_reader = study_reader.StudyReader('test_id', '', False, data_file=path, study_path=self.reader.study_path)
        temp_study = temp_reader.get_study()
        self.assertEqual("2", temp_study.subjects[0].headers["1"])

        # Use original file
        reader = study_reader.StudyReader('test_id', '', data_file=path, study_path=self.reader.study_path)
        not_temp_study = reader.get_study()
        self.assertEqual("111", not_temp_study.subjects[0].headers["NUMERO"])

    def test_folder_reading(self):
        """Tests reading all files from a folder"""
        loaded_study = self.reader.get_study()
        self.assertEqual(len(loaded_study.subjects), 6)

    def test_dafpath_reading(self):
        temp_reader = study_reader.StudyReader('test_id', '', False,
                                               data_file=os.path.join(self.reader.study_path, 'test_data.txt'),
                                               study_path=self.reader.study_path)
        subjects = temp_reader.get_subjects()
        header_info = study.HeaderInfo()
        for subject in subjects:
            subject.resolve_daf(header_info)
        relpath = os.path.relpath(subjects[1].data_file, self.reader.study_path)
        self.assertEqual(relpath, '1234.png')

    def test_temp_reading(self):
        study1 = self.reader.get_study()
        temp_reader = study_reader.StudyReader('test_id', '', False, study_path=self.reader.study_path)
        study2 = temp_reader.get_study()

        assert study1 == study2  # The version read from the temp file should be identical
        self.assertNotEqual(study1, "")

    def test_empty_subject(self):
        path = os.path.join(self.get_path_all(), "test_subject_no_data.txt")
        reader = study_reader.StudyReader('test_id', '', data_file=path, study_path=self.get_path_all())
        reader.get_study()  # Should not raise exceptions, this is a valid format for certain study types

    def test_no_subjects(self):
        path = os.path.join(self.get_path_all(), "test_no_subjects.txt")
        reader = study_reader.StudyReader('test_id', '', data_file=path, study_path=self.get_path_all())
        with pytest.raises(study_reader.NoSubjectsException):
            reader.get_study()

    def test_malformed_odt(self):
        """Tries to read an .odt file with a random section removed from the binary file"""
        path = os.path.join(self.get_path_all(), "malformed_odt.odt")
        path2 = os.path.join(self.get_path_all(), "malformed_odt2.odt")
        reader = study_reader.StudyReader('test_id', '', data_file=path, study_path=self.get_path_all())
        reader2 = study_reader.StudyReader('test_id', '', data_file=path2, study_path=self.get_path_all())

        with pytest.raises(odt_reader.ODTException):
            reader.get_study()

        with pytest.raises(odt_reader.ODTException):
            reader2.get_study()

    def test_heading_only_study(self):
        heading_only = os.path.join(self.get_path_all(), 'heading_only_test')
        reader = study_reader.StudyReader('test', '', study_path=heading_only)
        study_out = reader.get_study()
        self.assertEqual('111', study_out.subjects[0].get_header_value('NUMERO'))
        self.assertEqual('222', study_out.subjects[1].get_header_value('NUMERO'))
        self.assertEqual('333', study_out.subjects[2].get_header_value('NUMERO'))
