""" Tests the template generator.
"""
import pytest
import unittest
import os
import xml.etree.ElementTree as ET
from html.parser import HTMLParser
from kvalikirstu2 import study_reader
from kvalikirstu2 import data_archive_interface
from kvalikirstu2 import template_generator
from kvalikirstu2 import argument_parser
from kvalikirstu2 import header_scanner
from kvalikirstu2.localization import _


@pytest.fixture(name='testdir')
def get_testdir():
    return os.path.join(os.path.dirname(__file__), "test files", "valid", "data")


@pytest.fixture(name='settings')
def set_up_settings_class():
    return argument_parser.Settings()


@pytest.fixture(name='test_id')
def get_test_id():
    return 'test_id'


@pytest.fixture(name='store_args')
def store_test_args(settings, testdir):
    settings.parse_command_line([None, '--path', testdir, '--parse', '--overwrite_temp'])
    argument_parser.store_settings(settings.settings)


class HTMLTagCountParser(HTMLParser):
    """Makes sure that each starting tag has a matching ending tag."""
    def __init__(self, page):
        self.page = page
        self.tag_stack = []
        super().__init__()

    def handle_starttag(self, tag, attrs):
        self.tag_stack.append(tag)

    def handle_endtag(self, tag):
        if self.tag_stack and self.tag_stack[-1] == tag:
            self.tag_stack.pop()

    def check_matching_tags(self):
        if self.tag_stack:
            print("The contents of the tag stack are", self.tag_stack)
        return not self.tag_stack


class TestTemplateGenerator(unittest.TestCase):
    """Tests the template generator"""
    @pytest.fixture(autouse=True)
    def init_reader(self, testdir, test_id, store_args):
        self.args = argument_parser.get_args()
        self.reader = study_reader.StudyReader(testdir, test_id, "", study_path=os.path.join(os.path.dirname(__file__),
                                               "test files", "valid"))
        self.testdir = testdir

    def check_html(self, page):
        parser = HTMLTagCountParser(page)
        with open(page, 'r') as file_read:
            for line in file_read:
                parser.feed(line)
        return parser.check_matching_tags()

    def setUp(self):
        reader = study_reader.StudyReader(os.path.join(self.testdir, "test_data.txt"), 'test_id', '', False,
                                          study_path=self.testdir)
        lstudy = reader.get_study()
        header_info = header_scanner.get_header_info_for_study(self.testdir)
        header_info.header_enabled["Folder"] = True
        self.metadata = data_archive_interface.MetadataInformation('xxxx', 'citreq', '')
        template_generator.generate_index(self.testdir, lstudy, header_info, self.metadata)

    def _parse(self, filepath):
        return ET.parse(filepath)

    def test_name_generation(self):
        self.assertEqual('daF_1.html', template_generator._get_html_filepath('', 0, 1, self.metadata, ''))
        self.assertEqual('daF_01.html', template_generator._get_html_filepath('', 0, 2, self.metadata, ''))
        self.assertEqual('daF_012.html', template_generator._get_html_filepath('', 11, 3, self.metadata, ''))

    def test_lang_codes_subject(self):
        self.assertEqual('daF_1_fin.html', template_generator._get_html_filepath('', 0, 1, self.metadata, 'fin'))

    def test_lang_codes_all_subjects(self):
        self.assertEqual('daF_%s_fin.html' % _("all_subjects"),
                         template_generator._get_all_subjects_filepath('', self.metadata, 'fin'))

    def test_html(self):
        self.assertTrue(self.check_html(os.path.join(self.testdir, "index.html")))
        self.assertTrue(self.check_html(os.path.join(self.testdir, self.args.study_data_folder,
                        self.args.index_data_folder, "daF_%s.html" % _("all_subjects"))))
        self.assertTrue(self.check_html(os.path.join(self.testdir, self.args.study_data_folder,
                        self.args.index_data_folder, "daF_1.html")))
        self.assertTrue(self.check_html(os.path.join(self.testdir, self.args.study_data_folder,
                        self.args.index_data_folder, "daF_2.html")))
        self.assertTrue(self.check_html(os.path.join(self.testdir, self.args.study_data_folder,
                        self.args.index_data_folder, "daF_3.html")))


if __name__ == '__main__':
    unittest.main()
