""" Tests the header scanner.
"""

import unittest
import os
import tempfile
from kvalikirstu2 import argument_parser, utils
from tests import conftest


class TestUtils(unittest.TestCase):
    def setUp(self):
        argument_parser.Settings().parse_gui(['--max-backup-folder-size', '0'])

    def test_header_parsing(self):
        line = "header: value"
        header, value = utils.parse_header(line)
        self.assertEqual("header", header)
        self.assertEqual("value", value)

    def test_is_header_line(self):
        line = "header: value"
        self.assertTrue(utils.is_header_line(line))
        self.assertTrue(utils.is_header_line(line, {"header": True}))

        self.assertFalse(utils.is_header_line(line, {"header": False}))
        self.assertFalse(utils.is_header_line(line, {"other header": True}))

    def test_get_path_without_extension(self):
        path = os.path.join("folder", "folder", "data.txt")
        self.assertEqual(os.path.join("folder", "folder", "data"), utils.get_path_without_extension(path))

    def test_temp_files(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            temp_filepath = utils.get_temp_path(temp_dir, ".txt")
            with open(temp_filepath, mode='w'):
                pass
            self.assertEqual(temp_dir, utils.get_folder(temp_dir))
            self.assertEqual(temp_dir, utils.get_folder(temp_filepath))
            self.assertEqual([temp_filepath], utils.get_supported_files(temp_filepath))
            utils.delete_file_if_exists(temp_filepath)
            self.assertFalse(os.path.exists(temp_filepath))

    def test_number_of_digits(self):
        self.assertEqual(1, utils.number_of_digits(1))
        self.assertEqual(2, utils.number_of_digits(15))
        self.assertEqual(2, utils.number_of_digits(10))
        self.assertEqual(3, utils.number_of_digits(999))

    def test_header_with_questionmark(self):
        header = "A question?: an answer"
        self.assertTrue(utils.is_header_line(header))

    def test_too_long_header(self):
        header = '%s:test' % (257 * 'n')
        self.assertFalse(utils.is_header_line(header))

    def test_max_length_header(self):
        header = '%s:test' % (256 * 'n')
        self.assertTrue(utils.is_header_line(header))

    def test_header_with_colon(self):
        header = "header: colon:"
        self.assertTrue(utils.is_header_line(header))
        header_val, value = utils.parse_header(header)
        self.assertEqual('header', header_val)
        self.assertEqual('colon:', value)

    def test_file_size_on_empty_file(self):
        test_dir = conftest.get_test_dir()
        self.assertEqual(0, utils.get_filesize(os.path.join(test_dir, 'empty file.txt')))

    def test_file_size_on_nonempty_file(self):
        test_dir = conftest.get_test_dir()
        self.assertEqual(0.1, utils.get_filesize(os.path.join(test_dir, 'not-empty file.txt')))

    def test_get_pair(self):
        pair = ('test', 'asd')
        str_pair = str(pair)
        converted_key, converted_value = utils.get_pair_out_of_string(str_pair)
        converted = (converted_key, converted_value)
        self.assertEqual(pair, converted)

    def test_get_language_code(self):
        filepath = os.path.join('folder', 'path_fin.txt')
        self.assertEqual('fin', utils.get_language_code_from_path(filepath))

    def test_get_invalid_code(self):
        filepath = os.path.join('folder', 'path_fi.txt')
        self.assertEqual('', utils.get_language_code_from_path(filepath))

    def test_language_code_valid_list(self):
        filepath = os.path.join('folder', 'path_fin.txt')
        self.assertEqual('fin', utils.get_language_code_from_path(filepath, ['rus', 'fin', 'eng']))

    def test_language_code_invalid_list(self):
        filepath = os.path.join('folder', 'path_fin.txt')
        self.assertEqual('', utils.get_language_code_from_path(filepath, ['rus', 'eng']))

    def test_update_language_code(self):
        filepath = os.path.join('folder', 'path_fin.txt')
        self.assertEqual(os.path.join('folder', 'path_eng.txt'),
                         utils.add_language_code_to_path(filepath, 'eng', ['eng', 'fin']))

    def test_invalid_language_code_raises_value_error(self):
        with self.assertRaises(ValueError):
            utils.add_language_code_to_path('', 'a', ['b'])

    def test_pair_with_quotation_marks(self):
        pair = ('>(Sung to the tune', '"If You\'re Happy And You Know It Clap Your Hands")')
        string_pair = str(pair)
        parsed = utils.get_pair_out_of_string(string_pair)
        self.assertEqual(parsed, pair)

    def test_code_middle_path(self):
        test_path = os.path.join('test', 'test_fin', 'asd.txt')
        code = utils.get_language_code_from_path(test_path)
        self.assertFalse(code)

    def test_add_code_middle_path(self):
        test_path = os.path.join('test', 'test_fin', 'asd.txt')
        new_path = utils.add_language_code_to_path(test_path, 'eng', ['eng'])
        self.assertEqual(os.path.join('test', 'test_fin', 'asd_eng.txt'), new_path)
