""" Tests the file writing.
"""

import unittest
import tempfile
import os
from odf.opendocument import load
from kvalikirstu2 import argument_parser
from kvalikirstu2 import writer
from kvalikirstu2 import utils
from kvalikirstu2 import reader
from kvalikirstu2 import study
from kvalikirstu2 import odt_reader


def read_lines_from_file(path, discard_empty=True):
    file_reader = reader.get_reader(path)
    lines = []
    while file_reader.can_read():
        line = file_reader.read_line()
        if line or not discard_empty:
            lines.append(line)
    return lines


class CitReqMixin:
    def get_citreq_datadir(self):
        return os.path.join(self.get_citreq_dir(), "data")

    def get_citreq_dir(self):
        return os.path.join(os.path.dirname(__file__), "test files", "cit req test")

    def populate_citreq(self):
        path = self.get_citreq_datadir()
        utils.create_folder_if_not_exists(path)
        odt_path = utils.get_temp_path(path, ".odt")
        txt_path = utils.get_temp_path(path, ".txt")
        writer.write_empty_odt(odt_path)
        writer.write_txt(txt_path)

        return path, odt_path, txt_path

    def unpopulate_citreq(self):
        for filepath in utils.get_supported_files(self.get_citreq_datadir()):
            os.unlink(filepath)

    def read_lines_from_file(self, path, discard_empty=True):
        return read_lines_from_file(path, discard_empty)


class TestWriter(unittest.TestCase, CitReqMixin):
    def setUp(self):
        argument_parser.Settings().set_up_cli_settings([])

    def try_writing(self, start_text, added):
        temp_path = tempfile.gettempdir()
        path = utils.get_temp_path(temp_path, ".txt")
        with open(path, mode="w") as temp:
            temp.write(start_text)
        writer.add_text_to_file(path, path, added)
        with reader.get_reader(path) as file_reader:
            lines = []
            while file_reader.can_read():
                line = file_reader.read_line()
                if line:
                    lines.append(line)
        os.unlink(path)
        return lines

    def try_writing_odt(self, added):
        temp_path = tempfile.gettempdir()
        path = utils.get_temp_path(temp_path, ".odt")
        writer.write_empty_odt(path)

        for line in added:
            writer.add_text_to_file(path, path, line)
        lines = self.read_lines_from_file(path, False)
        os.unlink(path)
        return lines

    def test_writer(self):
        self.assertEqual(self.try_writing("test", "asd"), ["asd", "test"])
        self.assertEqual(self.try_writing("test", "asd\nasd"), ["asd", "asd", "test"])
        self.assertEqual(self.try_writing_odt(["asd", "fgh"]), ["fgh", "", "asd"])

    def test_reader_invalid_files(self):
        self.assertEqual(reader.get_reader("test.rtf"), None)
        self.assertEqual(reader.get_reader("test.png"), None)

    def test_citreq(self):
        citreq = "test"
        path, odt_path, txt_path = self.populate_citreq()
        writer.add_text_to_folder(path, citreq)
        self.assertEqual(self.read_lines_from_file(txt_path), [citreq])
        self.assertEqual(self.read_lines_from_file(odt_path), [citreq])
        self.unpopulate_citreq()

    def test_write_to_invalid_path(self):
        self.assertFalse(writer._should_write_line(None, ""))

    def test_remap_txt_headers(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test.txt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test1.txt")
        selected_headers = {'1': True, '2': True, '3': True, '4': True, '5': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c', '4': 'd', '5': 'e'}
        writer._rename_headers_in_txtfile(filepath, output_path, selected_headers, header_mapping, None, 'utf-8')

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line", lines[0])
        self.assertEqual("a: 1", lines[1])
        self.assertEqual("b: 2", lines[2])
        self.assertEqual("c: 3", lines[3])
        self.assertEqual("d: 4", lines[4])
        self.assertEqual("e: 5", lines[5])
        self.assertEqual("data", lines[6])

    def test_remap_odt_headers(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test.odt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test1.odt")
        selected_headers = {'1': True, '2': True, '3': True, '4': True, '5': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c', '4': 'd', '5': 'e'}
        header_value_mapping = {"('1', '1')": "2", "('2', '2')": "3"}
        writer._rename_headers_in_odtfile(filepath, output_path, selected_headers, header_mapping, header_value_mapping)

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line", lines[0])
        self.assertEqual("a: 2", lines[1])
        self.assertEqual("b: 3", lines[2])
        self.assertEqual("c: 3", lines[3])
        self.assertEqual("d: 4", lines[4])
        self.assertEqual("e: 5", lines[5])
        self.assertEqual("data", lines[6])

    def test_remap_txt_headers2(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test2.odt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test2_1.odt")
        selected_headers = {'1': True, '2': True, '3': True, '4': True, '5': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c', '4': 'd', '5': 'e'}
        header_value_mapping = {"('1', '1')": "a", "('2', '2')": "b"}
        writer._rename_headers_in_odtfile(filepath, output_path, selected_headers, header_mapping,
                                          header_value_mapping)

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line", lines[0])
        self.assertEqual("a: a", lines[1])
        self.assertEqual("b: b", lines[2])
        self.assertEqual("c: 3", lines[3])
        self.assertEqual("d: 4", lines[4])
        self.assertEqual("e: 5", lines[5])
        self.assertEqual("data1", lines[6])
        self.assertEqual("data2", lines[7])
        self.assertEqual("a: a", lines[8])
        self.assertEqual("b: b", lines[9])
        self.assertEqual("c: 3", lines[10])
        self.assertEqual("d: 4", lines[11])
        self.assertEqual("e: 5", lines[12])

    def test_remap_txt_headers3(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test2.txt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test2_1.txt")
        selected_headers = {'1': True, '2': True, '3': True, '555': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c', '555': 'a'}
        writer._rename_headers_in_txtfile(filepath, output_path, selected_headers, header_mapping, None, 'utf-8')

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line1", lines[0])
        self.assertEqual("a: 11", lines[1])
        self.assertEqual("b: 22", lines[2])
        self.assertEqual("c: 33", lines[3])
        self.assertEqual("line2", lines[4])
        self.assertEqual("data", lines[5])

    def test_remap_txt_headers_to_same(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test2.txt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test2_1.txt")
        selected_headers = {'1': True, '2': True, '3': True}
        header_mapping = {'1': 'a', '2': 'a', '3': 'a'}
        value_mapping = {"('1', '11')": "5"}
        writer._rename_headers_in_txtfile(filepath, output_path, selected_headers,
                                          header_mapping, value_mapping, 'utf-8')

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line1", lines[0])
        self.assertEqual("a: 5", lines[1])
        self.assertEqual("a: 22", lines[2])
        self.assertEqual("a: 33", lines[3])
        self.assertEqual("line2", lines[4])
        self.assertEqual("data", lines[5])

    def test_remap_odt_headers3(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test3.odt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test3_1.odt")
        selected_headers = {'1': True, '2': True, '3': True, '555': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c', '555': 'a'}
        writer._rename_headers_in_odtfile(filepath, output_path, selected_headers, header_mapping, None)

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line1", lines[0])
        self.assertEqual("a: 11", lines[1])
        self.assertEqual("b: 22", lines[2])
        self.assertEqual("c: 33", lines[3])
        self.assertEqual("line2", lines[4])
        self.assertEqual("data", lines[5])

    def test_remap_odt_headers_to_same(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test3.odt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test3_1.odt")
        selected_headers = {'1': True, '2': True, '3': True}
        header_mapping = {'1': 'a', '2': 'a', '3': 'a'}
        value_mapping = {"('1', '11')": "5"}
        writer._rename_headers_in_odtfile(filepath, output_path, selected_headers, header_mapping, value_mapping)

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("line1", lines[0])
        self.assertEqual("a: 5", lines[1])
        self.assertEqual("a: 22", lines[2])
        self.assertEqual("a: 33", lines[3])
        self.assertEqual("line2", lines[4])
        self.assertEqual("data", lines[5])

    def test_remap_odt_header_paragraph_detection(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test4.odt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test4_1.odt")
        selected_headers = {'1': True, '2': True, '3': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c'}
        writer._rename_headers_in_odtfile(filepath, output_path, selected_headers, header_mapping, None)

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("a: 11", lines[0])
        self.assertEqual("b: 22", lines[1])
        self.assertEqual("c: 33", lines[2])
        self.assertEqual("a: 11", lines[3])
        self.assertEqual("b: 22", lines[4])
        self.assertEqual("c: 33", lines[5])
        files = utils.natsorted_glob(os.path.join(os.path.dirname(__file__),
                                       "test files", "rename header test", "data", "**.orig"))
        self.assertFalse(files)

    def test_file_in_use_check(self):
        path = os.path.join(os.path.dirname(__file__), "test files", "rename recovery test")
        header_info = study.HeaderInfo()
        header_info.header_mapping = {'1': 'a', '2': 'b', '3': 'c'}
        with self.assertRaises(OSError):
            with open(os.path.join(os.path.dirname(__file__), "test files", "rename recovery test",
                                   "2.odt", "a")):
                writer.rename_headers_in_folder(path, header_info)
        lines1 = read_lines_from_file(os.path.join(os.path.dirname(__file__), "test files", "rename recovery test",
                                      "1.txt"))
        lines2 = read_lines_from_file(os.path.join(os.path.dirname(__file__), "test files", "rename recovery test",
                                      "3.txt"))

        self.assertEqual('1: a', lines1[0])
        self.assertEqual('2: b', lines1[1])
        self.assertEqual('3: c', lines1[2])
        self.assertEqual('data1', lines1[3])
        self.assertEqual('1: a', lines1[4])
        self.assertEqual('2: b', lines1[5])
        self.assertEqual('3: c', lines1[6])
        self.assertEqual('data2', lines1[7])

        self.assertEqual('1: a', lines2[0])
        self.assertEqual('2: b', lines2[1])
        self.assertEqual('3: c', lines2[2])
        self.assertEqual('data1', lines2[3])
        self.assertEqual('1: a', lines2[4])
        self.assertEqual('2: b', lines2[5])
        self.assertEqual('3: c', lines2[6])
        self.assertEqual('data2', lines2[7])

    def test_remap_odt_heading_only(self):
        filepath = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test5.odt")
        output_path = os.path.join(os.path.dirname(__file__), "test files", "rename header test", "test5_1.odt")
        selected_headers = {'1': True, '2': True, '3': True}
        header_mapping = {'1': 'a', '2': 'b', '3': 'c'}
        writer._rename_headers_in_odtfile(filepath, output_path, selected_headers, header_mapping, None)

        lines = self.read_lines_from_file(output_path)
        self.assertEqual("a: 11", lines[0])
        self.assertEqual("b: 22", lines[1])
        self.assertEqual("c: 33", lines[2])
        self.assertEqual("a: 11", lines[3])
        self.assertEqual("b: 22", lines[4])
        self.assertEqual("c: 33", lines[5])

    def test_replace_text_in_txt(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            path = os.path.join(temp_dir, 'txt.txt')
            with open(path, encoding='utf-8', mode='w') as file:
                file.write('testtest\ntest\n')
            writer.replace_in_folder(temp_dir, 'test', 'toast', encoding='utf-8')
            lines = self.read_lines_from_file(path)
            self.assertEqual(lines[0], 'toasttoast')
            self.assertEqual(lines[1], 'toast')

    def test_replace_text_in_odt(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            path = os.path.join(temp_dir, 'odt.odt')
            writer.write_empty_odt(path)
            writer.add_text_to_folder(temp_dir, 'test', False)
            writer.add_text_to_folder(temp_dir, 'testtest', False)
            writer.replace_in_folder(temp_dir, 'test', 'toast', encoding='utf-8')
            lines = self.read_lines_from_file(path)
            self.assertEqual(lines[0], 'toasttoast')
            self.assertEqual(lines[1], 'toast')

    def test_duplicate_detection(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            path = os.path.join(temp_dir, 'odt.odt')
            writer.write_empty_odt(path)
            writer.add_text_to_file(path, path, 'a')
            writer.add_text_to_folder(temp_dir, 'a', True)
            self.assertEqual(['a'], self.read_lines_from_file(path, discard_empty=False))
            writer.add_text_to_folder(temp_dir, 'b', True)
            self.assertEqual(['b', '', 'a'], self.read_lines_from_file(path, discard_empty=False))

    def test_add_multiline_text(self):
        with tempfile.TemporaryDirectory() as temp_dir:
            path = os.path.join(temp_dir, 'odt.odt')
            writer.write_empty_odt(path)
            writer.add_text_to_file(path, path, 'a\nb')
            lines = self.read_lines_from_file(path, discard_empty=False)
            self.assertEqual('a', lines[0])
            self.assertEqual('b', lines[1])
            self.assertEqual(2, len(lines))

            writer.add_text_to_file(path, path, 'c\nd')
            lines = self.read_lines_from_file(path, discard_empty=False)
            self.assertEqual('c', lines[0])
            self.assertEqual('d', lines[1])
            self.assertEqual('a', lines[3])
            self.assertEqual('b', lines[4])
            self.assertEqual(5, len(lines))
